/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: predecla.hxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: hr $ $Date: 2006/06/19 11:53:53 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifndef ARY_CPP_PREDECLA_HXX
#define ARY_CPP_PREDECLA_HXX



// USED SERVICES
	// BASE CLASSES
	// COMPONENTS
#include <ary/ids.hxx>
	// PARAMETERS
#include <ary/cpp/ca_type.hxx>


namespace ary
{
namespace cpp
{
	class UsedType;
	class Namespace;
	class Class;

	class ToNode_Namespace;
	class ToNode_Class;


class TypeOwnerNode
{
  public:
	virtual				~TypeOwnerNode() {}

	// OPERATIONS
	void     			Add_Type(
							const UsedType &   	i_rType );
	void				Add_LocalOwner(
							ToNode_Namespace & 	i_rNode_Nsp );
	void				Add_LocalOwner(
							ToNode_Class & 		i_rNode_Class );

	// INQUIRY
	Cid                 Id_Ce() const;
	const udmstri &		LocalName() const;
	const Type *      	Search_Type(
							const UsedType &    i_rType ) const;
	// ACCESS
	TypeOwnerNode *		Search_LocalOwner(
							const Namespace &   i_rNsp );
	TypeOwnerNode *		Search_LocalOwner(
							const Class &       i_rClass );
  protected:
	struct CheshireCat;

  private:
	CheshireCat &		my_Data() const;        // access_Data() const;
	virtual CheshireCat &
						access_Data() = 0;
	virtual void	   	do_Add_LocalOwner(
							ToNode_Namespace & 	i_rNode_Nsp ) = 0;
	virtual TypeOwnerNode *
						access_Search_LocalOwner(
							const Namespace &   i_rNsp ) = 0;
};


class ToNode_Namespace : public TypeOwnerNode
{
  public:
						ToNode_Namespace(
							const Namespace &	i_nRelatedNamespace,
							TypeOwnerNode *		i_pParent );
						~ToNode_Namespace();
  private:
	// Interface TypeOwnerNode:
	virtual CheshireCat &
						access_Data();
	virtual void	   	do_Add_LocalOwner(
							ToNode_Namespace & 	i_rNode_Nsp );
	virtual TypeOwnerNode *
						access_Search_LocalOwner(
							const Namespace &   i_rNsp );

	// Locals
	typedef std::map< udmstri, ToNode_Namespace * >	Index_Name2NspNode;

	// DATA
	Dyn<CheshireCat>	pNode;
	Index_Name2NspNode	aNamespace_Children;
};

class ToNode_Class : public TypeOwnerNode
{
  public:
						ToNode_Class(
							const Class &		i_nRelatedClass,
							TypeOwnerNode *		i_pParent );
						~ToNode_Class();
  private:
	// Interface TypeOwnerNode:
	virtual CheshireCat &
						access_Data();
	virtual void	   	do_Add_LocalOwner(
							ToNode_Namespace & 	i_rNode_Nsp );
	virtual TypeOwnerNode *
						access_Search_LocalOwner(
							const Namespace &   i_rNsp );
	// DATA
	Dyn<CheshireCat>	pNode;
};


// IMPLEMENTATION

inline TypeOwnerNode::CheshireCat &
TypeOwnerNode::my_Data() const
	{ return const_cast< TypeOwnerNode* >(this)->access_Data(); }

inline void
TypeOwnerNode::Add_LocalOwner( ToNode_Namespace & i_rNode_Nsp )
	{ do_Add_LocalOwner( i_rNode_Nsp ); }

inline TypeOwnerNode *
TypeOwnerNode::Search_LocalOwner( const Namespace & i_rNsp )
	{ return access_Search_LocalOwner( i_rNsp ); }




}   // namespace cpp
}   // namespace ary



#endif

