/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: defdescr.hxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: rt $ $Date: 2005/09/07 18:20:35 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifndef ADC_CPP_DEFDESCR_HXX
#define ADC_CPP_DEFDESCR_HXX



// USED SERVICES
	// BASE CLASSES
	// COMPONENTS
	// PARAMETERS


namespace cpp
{

/** Describes a C/C++ #define statement. May be a define or a macro, for which
    two cases the two different constructors are to be used.

    This class is used by cpp::PreProcessor.
*/
class DefineDescription
{
  public:
    enum E_DefineType
    {
        type_define,
        type_macro
    };
    typedef StringVector str_vector;

                        DefineDescription(      /// Used for: #define DEFINE xyz
                            const udmstri &     i_sName,
                            const str_vector &  i_rDefinition );
                        DefineDescription(      /// Used for: #define MACRO(...) abc
                            const udmstri &     i_sName,
                            const str_vector &  i_rParams,
                            const str_vector &  i_rDefinition );
                        ~DefineDescription();

    /// Only vaild if (eDefineType == type_define) else returns "".
    void                GetDefineText(
                            csv::StreamStr &    o_rText ) const;

    /// Only vaild if (eDefineType == type_macro) else returns "".
    void                GetMacroText(
                            csv::StreamStr &    o_rText,
                            const StringVector &
                                                i_rGivenArguments ) const;

    uintt               ParamCount() const;
    E_DefineType        DefineType() const;

  private:
    // DATA
    udmstri             sName;
    str_vector          aParams;
    str_vector          aDefinition;
    E_DefineType        eDefineType;
};



// IMPLEMENTATION

inline uintt
DefineDescription::ParamCount() const
    { return aParams.size(); }
inline DefineDescription::E_DefineType
DefineDescription::DefineType() const
    { return eDefineType; }


}   // end namespace cpp



#endif


