/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: schview.hxx,v $
 *
 *  $Revision: 1.10 $
 *
 *  last change: $Author: ihi $ $Date: 2006/11/14 14:52:24 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifndef _SCH_SCHVIEW_HXX
#define _SCH_SCHVIEW_HXX

#ifndef _SVDEDXV_HXX //autogen
#include <svx/svdedxv.hxx>
#endif
#ifndef _E3D_VIEW3D_HXX //autogen
#include <svx/view3d.hxx>
#endif

#ifndef _SVDPAGE_HXX //autogen
#include <svx/svdpage.hxx>
#endif
#ifndef _TRANSFER_HXX //autogen
#include <svtools/transfer.hxx>
#endif


#ifndef _CHTMODEL_HXX
#include "chtmodel.hxx"
#endif

#include "docshell.hxx"

class SchViewShell;
class SchPage;
class SchWindow;
class ChartModel;
class SchDragServerRef;
class E3dScene;
class Graphic;

namespace com { namespace sun { namespace star { namespace datatransfer {
	class XTransferable;
}}}}

DECLARE_LIST(E3dLogicalObjList, E3dObject*);

/*************************************************************************
|*
|* View
|*
\************************************************************************/

class SchView : public E3dView
{
 protected:
	ChartModel*   pDoc;
	SchChartDocShell*   pDocSh;
	SchViewShell*       pViewSh;
	BOOL                bDragActive;
	USHORT              nLogicalMarked;
	USHORT              nLogicalEntered;

	Timer aTimer;
	static void      DeleteChartUserData(ChartModel& rDoc);
	void             Construct();
//  	SchDragServerRef CreateDataObject(SchView* pView, const Point& rDragPos);
//  	SchDragServerRef CreateDataObject(ChartModel* pDocument);
	void                SetMarkHandles();

 public:

	//void NotifySelection();
	DECL_LINK(NotifySelection,void*);

	SchView(SchChartDocShell* pDocSh, OutputDevice* pOutDev, SchViewShell* pShell);
	SchView(ChartModel* pDoc, OutputDevice* pOutDev);
	virtual ~SchView();

	virtual BOOL DoCut(Window* pWindow=NULL);
	virtual void DoCopy(Window* pWindow=NULL);
	virtual void DoPaste(Window* pWindow=NULL);

	sal_Bool SdrBeginTextEdit(
		SdrObject* pObj, SdrPageView* pPV = 0L, Window* pWin = 0L, 
		sal_Bool bIsNewObj = sal_False, SdrOutliner* pGivenOutliner = 0L, 
		OutlinerView* pGivenOutlinerView = 0L, 
		sal_Bool bDontDeleteOutliner = sal_False, sal_Bool bOnlyOneView = sal_False);
	SdrEndTextEditKind SdrEndTextEdit(sal_Bool bDontDeleteReally = sal_False);

	BOOL IsChartObjSelected();
	long GetSelectedRow() const;
	BOOL CanDeleteMarkedObjects();
	BOOL DeleteMarkedObjects(const String& rUndoStr);
	BOOL CanSetDataAttr();
	void PutMarkedToPos(ULONG nObjPos);
	void MovMarkedToPos(ULONG nObjPos);

	String GetContext();
	String GetContextType();

	ChartModel& GetDoc() const { return *pDoc; }

	BOOL BeginDrag( Window* pWindow, Point aStartPos );

	// clipboard / drag and drop methods using XTransferable
	sal_Bool InsertData( TransferableDataHelper& rDataHelper, const Point& rPos, BOOL bCopy, ULONG nFormat = 0 );
	::com::sun::star::uno::Reference< ::com::sun::star::datatransfer::XTransferable > CreateDragDropDataObject( SchView* pView, Window& rWindow, const Point& rDragPos );
	::com::sun::star::uno::Reference< ::com::sun::star::datatransfer::XTransferable > CreateClipboardDataObject();
	::com::sun::star::uno::Reference< ::com::sun::star::datatransfer::XTransferable > CreateSelectionDataObject( SchView* pWorkView, Window& rWindow );

	void UpdateSelectionClipboard( BOOL bForceDeselect );

	BOOL InsertGraphic( const Graphic& rGraphic, const Point& rPos );

	BOOL IsActive() const { return bDragActive; }

	virtual void InvalidateOneWin(Window& rWin, const Rectangle& rRect);

	void       SetDefault();

	virtual BOOL NotifyNewUndoAction();
	virtual void MarkListHasChanged();

	FASTBOOL    EnterMarkedGroup();
	void		LeaveOneGroup();
	void		LeaveAllGroup();

	FASTBOOL	IsLogicalGroupMarked() { return (nLogicalMarked != 0); }
	FASTBOOL	IsLogicalGroupEntered() { return (nLogicalEntered != 0); }
	void		MarkLogicalGroup();

	BOOL        PickObj(const Point& rPnt, short nTol, SdrObject*& prObj,
						SdrPageView*& prPV,ULONG nOptions=0) const;

    // redefinition of some SdrMarkView methods that are unfortunately not virtual
    BOOL        MarkObj(const Point& rPnt, short nTol=-2, BOOL bToggle=FALSE, BOOL bDeep=FALSE);
    void        MarkObj(SdrObject* pObj, SdrPageView* pPV, BOOL bUnmark=FALSE, BOOL bImpNoSetMarkHdl=FALSE);

	BOOL        IsMarkedHit(const Point& rPnt, short nTol=-2) const;

	E3dLogicalObjList GetLogicalObjList();
	SchChartDocShell* GetDocShell() const { return pDocSh; }

	// drag and drop
	virtual sal_Int8 AcceptDrop( const AcceptDropEvent& rEvt, SchWindow* pWin );
	virtual sal_Int8 ExecuteDrop( const ExecuteDropEvent& rEvt, SchWindow* pWin );

    Window* GetWindow() const;

    virtual void Notify( SfxBroadcaster& rBC, const SfxHint& rHint );

private:
    // #117447# old SdrMarkView::PickObj() method before fix for #110988#
    BOOL ChartPickObj(
        const Point& rPnt, short nTol,
        SdrObject*& rpObj, SdrPageView*& rpPV,
        ULONG nOptions) const;

    // redefinition of some SdrMarkView methods that are unfortunately not virtual
    SdrObject* ImpCheckObjHit(const Point& rPnt, USHORT nTol, SdrObject* pObj, SdrPageView* pPV, ULONG nOptions, const SetOfByte* pMVisLay) const;
    SdrObject* ImpCheckObjHit(const Point& rPnt, USHORT nTol, SdrObjList* pOL, SdrPageView* pPV, ULONG nOptions, const SetOfByte* pMVisLay, SdrObject*& rpRootObj) const;
    USHORT ImpGetHitTolLogic(short nHitTol, const OutputDevice* pOut) const;
};

#endif		// _SCH_SCHVIEW_HXX
