/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: chtmod3d.cxx,v $
 *
 *  $Revision: 1.26 $
 *
 *  last change: $Author: ihi $ $Date: 2006/11/14 14:54:33 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

// MARKER(update_precomp.py): autogen include statement, do not remove
#include "precompiled_sch.hxx"

#define RAD2CDEG(fAngle) ( (long)(((fAngle)*18000.0/F_PI)+36000.0)%36000 )

#ifndef _SVDVIEW_HXX //autogen
#include <svx/svdview.hxx>
#endif
#ifndef _SVDPAGV_HXX //autogen
#include <svx/svdpagv.hxx>
#endif

#include <svx/obj3d.hxx>

#ifndef _SCHATTR_HXX
#include "schattr.hxx"
#endif
#define ITEMID_ADJUST EE_PARA_JUST
#include <svx/adjitem.hxx>

#ifndef _SVX_CHRTITEM_HXX //autogen
#define ITEMID_CHARTDATADESCR	SCHATTR_DATADESCR_DESCR
#define ITEMID_CHARTTEXTORIENT	SCHATTR_TEXT_ORIENT
#include <svx/chrtitem.hxx>
#endif

#ifndef _SCH_OBJADJ_HXX
#include  "objadj.hxx"
#endif
#ifndef _CHTMODEL_HXX
#include <chtmodel.hxx>
#include <globfunc.hxx>
#endif
#ifndef _SCH_SCHRESID_HXX
#include "schresid.hxx"
#endif
#ifndef _SCH_DATAROW_HXX
#include "datarow.hxx"
#endif
#ifndef _SCH_DATAPOIN_HXX
#include "datapoin.hxx"
#endif
#ifndef _SCH_SCHGROUP_HXX
#include "schgroup.hxx"
#endif
#ifndef _SCH_AXISID_HXX
#include "axisid.hxx"
#endif
#ifndef _CHTSCENE_HXX
#include "chtscene.hxx"
#endif

#undef	ITEMID_COLOR		//	Defined in svx3ditems.hxx
#define ITEMID_COLOR       EE_CHAR_COLOR
#ifndef _SVX_COLRITEM_HXX //autogen wg. SvxColorItem	// #60999#
#include <svx/colritem.hxx>
#endif

#ifndef _E3D_POLYGON3D_HXX
#include <svx/polygn3d.hxx>
#endif
#ifndef _SVDOCIRC_HXX //autogen
#include <svx/svdocirc.hxx>
#endif
#include "math.h"
#include "glob.hrc"
#include "float.h"

#ifndef _ZFORLIST_HXX //autogen
#ifndef _ZFORLIST_DECLARE_TABLE
#define _ZFORLIST_DECLARE_TABLE
#endif
#include <svtools/zforlist.hxx>
#endif
#ifndef _SVDOPATH_HXX //autogen
#include <svx/svdopath.hxx>
#endif
#ifndef _SVX_XLNWTIT_HXX //autogen
#include <svx/xlnwtit.hxx>
#endif

#include "pairs.hxx"
#include "chmod3d.hxx"
#include "chaxis.hxx"
#include "chdescr.hxx"
#define FIXED_SIZE_FOR_3D_CHART_VOLUME			(10000)

#ifndef _SVX3DITEMS_HXX
#include <svx/svx3ditems.hxx>
#endif

#ifndef _BGFX_POLYGON_B3DPOLYGON_HXX
#include <basegfx/polygon/b3dpolygon.hxx>
#endif

#ifndef _BGFX_RANGE_B2DRANGE_HXX
#include <basegfx/range/b2drange.hxx>
#endif

#ifndef _BGFX_POLYGON_B2DPOLYGONTOOLS_HXX
#include <basegfx/polygon/b2dpolygontools.hxx>
#endif

/*************************************************************************
|*
|* Erzeugt alle 3d-Achsentitel und setzt sie sofort ins Diagramm
|* Positioniert werden sie von der Chartszene
|*
\************************************************************************/

void ChartModel::CreateAndInsert3DAxesTitles (Rectangle &rRect, BOOL bSwitchColRow)
{
	SdrTextObj *pXAxisTitleObj = NULL;
	SdrTextObj *pYAxisTitleObj = NULL;
	SdrTextObj *pZAxisTitleObj = NULL;

	if (bShowXAxisTitle)
	{
		pXAxisTitleObj = CreateTitle (pXAxisTitleAttr, CHOBJID_DIAGRAM_TITLE_X_AXIS,
												   bSwitchColRow,aXAxisTitle,
												   FALSE, &eAdjustXAxesTitle);
		if (GetAdjustMarginsForXAxisTitle())
		{
			if (bSwitchColRow)
				rRect.Left() += GetOutputSize(*pXAxisTitleObj).Width() + 200;
			else
				rRect.Bottom() -= GetOutputSize(*pXAxisTitleObj).Height() + 200;
		}
	}
	if (bShowYAxisTitle)
	{
		pYAxisTitleObj = CreateTitle (pYAxisTitleAttr, CHOBJID_DIAGRAM_TITLE_Y_AXIS,
												   bSwitchColRow, aYAxisTitle,
												   TRUE, &eAdjustYAxesTitle);
		if (GetAdjustMarginsForYAxisTitle())
		{
			if (bSwitchColRow)
				rRect.Bottom() -= GetOutputSize(*pYAxisTitleObj).Height() + 200;
			else
				rRect.Left() += GetOutputSize(*pYAxisTitleObj).Width() + 200;
		}
	}
	if (bShowZAxisTitle)
	{
		pZAxisTitleObj = CreateTitle (pZAxisTitleAttr, CHOBJID_DIAGRAM_TITLE_Z_AXIS,
			bSwitchColRow,aZAxisTitle, FALSE, &eAdjustZAxesTitle);
		if( GetAdjustMarginsForZAxisTitle())
        {
            rRect.Right() -= GetOutputSize(*pZAxisTitleObj).Width() + 200;
        }
	}

	SdrPage* pPage=GetPage(0);
	if (pXAxisTitleObj) pPage->NbcInsertObject( pXAxisTitleObj );
	if (pYAxisTitleObj) pPage->NbcInsertObject( pYAxisTitleObj );
	if (pZAxisTitleObj) pPage->NbcInsertObject( pZAxisTitleObj );

    // this should resize the scene according to the space needed
    // by the titles. However this results in strange effects like
    // constant resizing until the scene has 0 size and also the title
    // objects disappear when being moved (?).
//     GetScene()->NbcSetSnapRect( rRect );
}

/*************************************************************************
|*
|* 3D-Rueckwaende mit Unterteilung und Beschriftung erzeugen
|*
\************************************************************************/
void ChartModel::Create3DBackplanes (Rectangle		&rRect,
									 ::basegfx::B3DPoint	aPos,
									 ::basegfx::B3DPoint	aSizeVec,
									 ChartScene     &rScene,
									 BOOL           bPartDescr,
									 BOOL           bXLogarithm,
									 BOOL           bYLogarithm,
									 BOOL           bZLogarithm,
									 USHORT         eStackMode,
									 BOOL           bPercent,
									 BOOL           bFlatChart,
									 BOOL           bSwitchColRow)
{
	//vorlaeufiger Hack, bis logarithmus-Ueberpruefung eingebaut (autokorrektur):
	pChartXAxis->GetItemSet()->Put(SfxBoolItem(SCHATTR_AXIS_LOGARITHM,bXLogarithm));
	pChartYAxis->GetItemSet()->Put(SfxBoolItem(SCHATTR_AXIS_LOGARITHM,bYLogarithm));
	pChartZAxis->GetItemSet()->Put(SfxBoolItem(SCHATTR_AXIS_LOGARITHM,bZLogarithm));

	String     aNumStr;

	E3dObject* pXGridMainGroup = bShowXGridMain
											 ? Create3DObject (CHOBJID_DIAGRAM_X_GRID_MAIN_GROUP)
											 : 0;
	E3dObject* pYGridMainGroup = bShowYGridMain
											 ? Create3DObject (CHOBJID_DIAGRAM_Y_GRID_MAIN_GROUP)
											 : 0;
	E3dObject* pZGridMainGroup = bShowZGridMain
											 ? Create3DObject (CHOBJID_DIAGRAM_Z_GRID_MAIN_GROUP)
											 : 0;
	E3dObject* pXGridHelpGroup = bShowXGridHelp
											 ? Create3DObject (CHOBJID_DIAGRAM_X_GRID_HELP_GROUP)
											 : 0;
	E3dObject* pYGridHelpGroup = bShowYGridHelp
											 ? Create3DObject (CHOBJID_DIAGRAM_Y_GRID_HELP_GROUP)
											 : 0;
	E3dObject* pZGridHelpGroup = bShowZGridHelp
											 ? Create3DObject (CHOBJID_DIAGRAM_Z_GRID_HELP_GROUP)
											 : 0;
	E3dObject* pXAxisGroup     = pChartXAxis->IsVisible()|| pChartXAxis->HasDescription()  //#47500#
											 ? Create3DAxisObj( CHOBJID_DIAGRAM_X_AXIS )
											 : 0;
	E3dObject* pYAxisGroup     = pChartYAxis->IsVisible() || pChartYAxis->HasDescription() //#47500#
											 ? Create3DAxisObj( CHOBJID_DIAGRAM_Y_AXIS )
											 : 0;
	E3dObject* pZAxisGroup     = pChartZAxis->IsVisible() || pChartZAxis->HasDescription() //#47500#
											 ? Create3DAxisObj( CHOBJID_DIAGRAM_Z_AXIS )
											 : 0;

	pChartXAxis->Initialise(rRect,bSwitchColRow,eStackMode,bPercent,FALSE);
	pChartYAxis->Initialise(rRect,bSwitchColRow,eStackMode,bPercent,FALSE);
	pChartZAxis->Initialise(rRect,bSwitchColRow,eStackMode,bPercent,FALSE);

	pChartXAxis->CalcValueSteps();
	pChartYAxis->CalcValueSteps();
	pChartZAxis->CalcValueSteps();

//Vorlaeufige Notloesung, entsprechende Stellen sollen noch geaendert werden, daher bitte kein Search&Repleace statt der #defs!
#define	fOriginY		pChartYAxis->GetOrigin()
#define fMinValueY		pChartYAxis->GetMin()
#define fMaxValueY		pChartYAxis->GetMax()
#define fValueStepMainY pChartYAxis->GetStep()
#define fValueStepHelpY pChartYAxis->GetHelpStep()

	//ToDo: vorlaeufig
	SfxItemSet* pXAxisAttr=&GetAttr(CHOBJID_DIAGRAM_X_AXIS);
	SfxItemSet* pYAxisAttr=&GetAttr(CHOBJID_DIAGRAM_Y_AXIS);
	SfxItemSet* pZAxisAttr=&GetAttr(CHOBJID_DIAGRAM_Z_AXIS);

	SvxChartTextOrient eDescrOrient = ((const SvxChartTextOrientItem&)pXAxisAttr->Get(SCHATTR_TEXT_ORIENT)).GetValue();
	SvxChartTextOrient eValueOrient = ((const SvxChartTextOrientItem&)pYAxisAttr->Get(SCHATTR_TEXT_ORIENT)).GetValue();

	Size aMaxValueSizeY = pChartYAxis->CalcMaxTextSize(eValueOrient);
	Size aMaxValueSizeX = pChartXAxis->CalcMaxTextSize(eDescrOrient);

	Size aMaxDescrSizeY = CalcMaxDescrSize( TRUE,  eValueOrient, bPercent, CHAXIS_AXIS_Y );
	Size aMaxDescrSizeX = CalcMaxDescrSize( FALSE, eDescrOrient, bPercent, CHAXIS_AXIS_X );

	short   nV, i;

	long   nRowCnt     = bFlatChart
							  ? 1
							  : GetRowCount();
	long   nBackColCnt = (bPartDescr || GetColCount() > 1)
							  ? GetColCount()
							  : GetColCount() + 1;
	long   nColumnCnt  = bPartDescr
							  ? nBackColCnt
							  : nBackColCnt - 1;


	// FG: Diese Variable ist ein reiner Zwischenspeicher damit in der Chartscene bei FitInSnapRect
	//     die Achsentitel gemaess dieses Parameters gesetzt werden koennen.
	bSwitch3DColRow = bSwitchColRow;

	long nMaxTextWidth = 0;

	aXDescrList.Clear();
	aYDescrList.Clear();
	aZDescrList.Clear();

	E3dDefaultAttributes aDefltAttr3D;

	::basegfx::B3DHomMatrix aShift;
	const ::basegfx::B3DPoint aTransVec(-(aSizeVec/500.0));
	aShift.translate(aTransVec.getX(), aTransVec.getY(), aTransVec.getZ());//Waende etwas verschieben...

	const double fFloorWidth = 100.0;
	const double fWallWith	 = 50.0;	// BM: unused for now
	
	for (nV = 0; nV < 3; nV++)
	{
		switch (nV)
		{
			case 0:         // X-Achse , XY-Ebene (WALL hinten)
			case 1:         // Y-Achse , YZ-Ebene (WALL seitlich)
			{
				::basegfx::B3DPolygon aRect3D;

				aRect3D.append(aPos);
				if(!nV) aPos.setX(aPos.getX() + aSizeVec.getX()); else aPos.setY(aPos.getY() + aSizeVec.getY()); //nV=0: X(), sonst Y()
				aRect3D.append(aPos);
				if(!nV) aPos.setY(aPos.getY() + aSizeVec.getY()); else aPos.setZ(aPos.getZ() + aSizeVec.getZ()); //nV=0: Y(), sonst Z()
				aRect3D.append(aPos);
				if(!nV) aPos.setX(aPos.getX() - aSizeVec.getX()); else aPos.setY(aPos.getY() - aSizeVec.getY()); //nV=0: X(), sonst Y()
				aRect3D.append(aPos);
				if(!nV) aPos.setY(aPos.getY() - aSizeVec.getY()); else aPos.setZ(aPos.getZ() - aSizeVec.getZ()); //nV=0: Y(), sonst Z()

				E3dPolygonObj *pWallObj = new SchE3dPolygonObj(aDefltAttr3D, ::basegfx::B3DPolyPolygon(aRect3D));
				pWallObj->SetModel (this);
				pWallObj->InsertUserData(new SchObjectId(CHOBJID_DIAGRAM_WALL));
				rScene.Insert3DObj(pWallObj);

//-/				pWallObj->NbcSetAttributes(*pDiagramWallAttr, FALSE);
				pWallObj->SetMergedItemSet(*pDiagramWallAttr);

				pWallObj->NbcSetTransform(aShift);

				if (!nV)
				{
					// X-Achse zeichnen
					::basegfx::B3DPoint aLine3D[2] = { aRect3D.getB3DPoint(0L), aRect3D.getB3DPoint(1L) };

					//#47500#
					if (pXAxisGroup && pChartXAxis->IsVisible())
					{
						::basegfx::B3DPolygon aTempPoly;
						aTempPoly.append(aLine3D[0]);
						aTempPoly.append(aLine3D[1]);
						Create3DPolyObject (pXAxisAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_LINE, pXAxisGroup);
					}

					if( pYGridMainGroup || pYGridHelpGroup )
						// Y-Anteile
						if (bSwitchColRow)
						{
							::basegfx::B3DPoint aLine3D[2] = { aRect3D.getB3DPoint(0L), aRect3D.getB3DPoint(1L) };
							double fStepMainY  = aSizeVec.getY () / nColumnCnt;
							double fStepHelpY  = fStepMainY / 2.0;

							// hauptgitter auf der X-Ebene, parallel zur X-Achse
							for (i = 0; i <= nColumnCnt; i++)
							{
								const double fValue(aPos.getY() + fStepMainY * i);
								aLine3D[0].setY(fValue);
								aLine3D[1].setY(fValue);
								if( pYGridMainGroup )
								{
									::basegfx::B3DPolygon aTempPoly;
									aTempPoly.append(aLine3D[0]);
									aTempPoly.append(aLine3D[1]);
									Create3DPolyObject(pYGridMainAttr, new SchE3dPolygonObj(aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Y_GRID_MAIN, pYGridMainGroup);
								}

								// hilfsgitter auf der X-Ebene, parallel zur X-Achse
								if (pYGridHelpGroup && (i < nColumnCnt))
								{
									aLine3D[0].setY(aLine3D[0].getY() + fStepHelpY);
									aLine3D[1].setY(aLine3D[1].getY() + fStepHelpY);
									::basegfx::B3DPolygon aTempPoly;
									aTempPoly.append(aLine3D[0]);
									aTempPoly.append(aLine3D[1]);
									Create3DPolyObject (pYGridHelpAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Y_GRID_HELP, pYGridHelpGroup);
								}
							}
						}
						else
						{
							aLine3D [0] = aRect3D.getB3DPoint(0L);
							aLine3D [1] = aRect3D.getB3DPoint(3L);

							long nStepMainX  = (long) aSizeVec.getX () / nColumnCnt;
							long nStepHelpX  = nStepMainX / 2;

							// hauptgitter auf der X-Ebene, parallel zur Y-Achse
							for (i = 0; i <= nColumnCnt; i++)
							{
								const double fValue(aPos.getX() + nStepMainX * i);
								aLine3D[0].setX(fValue);
								aLine3D[1].setX(fValue);
								if( pYGridMainGroup )
								{
									::basegfx::B3DPolygon aTempPoly;
									aTempPoly.append(aLine3D[0]);
									aTempPoly.append(aLine3D[1]);
									Create3DPolyObject (pYGridMainAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Y_GRID_MAIN, pYGridMainGroup);
								}

								// hilfsgitter auf der X-Ebene, parallel zur Y-Achse
								if (pYGridHelpGroup && (i < nColumnCnt))
								{
									aLine3D[0].setX(aLine3D[0].getX() + nStepHelpX);
									aLine3D[1].setX(aLine3D[1].getX() + nStepHelpX);
									::basegfx::B3DPolygon aTempPoly;
									aTempPoly.append(aLine3D[0]);
									aTempPoly.append(aLine3D[1]);
									Create3DPolyObject (pYGridHelpAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Y_GRID_HELP, pYGridHelpGroup);
								}
							}
						}
				}
				else
				{
					// Y-Achse zeichnen
					::basegfx::B3DPoint aLine3D[2] = { aRect3D.getB3DPoint(0L), aRect3D.getB3DPoint(1L) };

					// Z-Anteile
					//#47500#
					if (pYAxisGroup && pChartYAxis->IsVisible()) 
					{
						::basegfx::B3DPolygon aTempPoly;
						aTempPoly.append(aLine3D[0]);
						aTempPoly.append(aLine3D[1]);
						Create3DPolyObject (pYAxisAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_LINE, pYAxisGroup);
					}

					// hauptgitter auf der Z-Ebene, parallel zur Z-Achse
					if( pZGridMainGroup || pZGridHelpGroup )
					{
						::basegfx::B3DPoint aLine3D[2] = { aRect3D.getB3DPoint(2L), aRect3D.getB3DPoint(3L) };
						long     nStepMainZ  = (long) aSizeVec.getZ () / nRowCnt;
						long     nStepHelpZ  = nStepMainZ / 2;

						BOOL bCreateGridLine = ( (aLine3D[ 0 ].getX() != aLine3D[ 1 ].getX()) ||
												 (aLine3D[ 0 ].getY() != aLine3D[ 1 ].getY()) );
						// Z() values become equal in the for loop
						// => start and end points would be equal if !bCreateGridLine

						for (i = 0; i <= nRowCnt; i++)
						{
							const double fValue(aPos.getZ() + nStepMainZ * i);
							aLine3D[0].setZ(fValue);
							aLine3D[1].setZ(fValue);
							if( pZGridMainGroup && bCreateGridLine )
							{
								::basegfx::B3DPolygon aTempPoly;
								aTempPoly.append(aLine3D[0]);
								aTempPoly.append(aLine3D[1]);
								Create3DPolyObject( pZGridMainAttr, new SchE3dPolygonObj( aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Z_GRID_MAIN, pZGridMainGroup );
							}

							// hilfsgitter auf der Z-Ebene, parallel zur Z-Achse
							if (pZGridHelpGroup && (i < nRowCnt))
							{
								aLine3D[0].setZ(aLine3D[0].getZ() + nStepHelpZ);
								aLine3D[1].setZ(aLine3D[1].getZ() + nStepHelpZ);
								::basegfx::B3DPolygon aTempPoly;
								aTempPoly.append(aLine3D[0]);
								aTempPoly.append(aLine3D[1]);
								if( bCreateGridLine )
									Create3DPolyObject( pZGridHelpAttr, new SchE3dPolygonObj( aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Z_GRID_HELP, pZGridHelpGroup );
							}
						}
					}
				}

				::basegfx::B3DPoint aLine3D[2] = { aRect3D.getB3DPoint(0L), aRect3D.getB3DPoint((nV || !nV && bSwitchColRow)
																   ? 3L
																   : 1L) };
				double     fAct        = fMinValueY;

				SfxItemSet aYTextAttr ((const SfxItemSet &) *pYAxisAttr);
				aYTextAttr.Put(XLineStyleItem (XLINE_NONE));
				aYTextAttr.Put(XLineWidthItem (0));

				// create Y grid. Attention: Some variables are called XGrid...
				if (fMinValueY != fMaxValueY)
					while (fAct <= fMaxValueY)
					{
						double fFact = pChartYAxis->CalcFact(fAct);

						if (bSwitchColRow)
						{
							// create major gridline
							aLine3D[0].setX(fFact * aSizeVec.getX());
							aLine3D[1].setX(fFact * aSizeVec.getX());

							if (pXGridMainGroup)
							{
								::basegfx::B3DPolygon aTempPoly;
								aTempPoly.append(aLine3D[0]);
								aTempPoly.append(aLine3D[1]);
								Create3DPolyObject(pXGridMainAttr, new SchE3dPolygonObj(aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true),CHOBJID_DIAGRAM_X_GRID_MAIN,pXGridMainGroup);
							}

							if (pYAxisGroup && pChartYAxis->HasDescription() && nV)
							{
								Color *pTextColor = NULL;
								SvxColorItem *pOldTextColor = NULL;

								pNumFormatter->GetOutputString((bPercent)?fAct/100.0:fAct,
									GetNumFmt(CHOBJID_DIAGRAM_Y_AXIS,bPercent), aNumStr, &pTextColor);

								if( pTextColor )			// BM #60999#
								{
									pOldTextColor = (SvxColorItem*)(aYTextAttr.GetItem( EE_CHAR_COLOR ));
									aYTextAttr.Put(SvxColorItem( *pTextColor ));
								}

								SdrTextObj *pTextObj = CreateTextObj(CHOBJID_TEXT, Point (),aNumStr,
									aYTextAttr, FALSE,CHADJUST_TOP_RIGHT);
								
								if( pOldTextColor )
								{
									aYTextAttr.Put( *pOldTextColor );
								}

								E3dLabelObj *pE3DLabel = new E3dLabelObj(aLine3D[1],pTextObj);
								aYDescrList.Insert(pE3DLabel,LIST_APPEND);
							}
						}
						else
						{
							// create major gridline
							aLine3D[0].setY(fFact * aSizeVec.getY());
							aLine3D[1].setY(fFact * aSizeVec.getY());

							if(pXGridMainGroup)
							{
								::basegfx::B3DPolygon aTempPoly;
								aTempPoly.append(aLine3D[0]);
								aTempPoly.append(aLine3D[1]);
								Create3DPolyObject(pXGridMainAttr,new SchE3dPolygonObj(aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true),CHOBJID_DIAGRAM_X_GRID_MAIN, pXGridMainGroup);
							}

							// Y-Achsenwerte
							if (pYAxisGroup && pChartYAxis->HasDescription() && nV)
							{
								Color *pTextColor = NULL;
								SvxColorItem *pOldTextColor = NULL;

								pNumFormatter->GetOutputString((bPercent)?fAct/100.0:fAct,
									GetNumFmt(CHOBJID_DIAGRAM_Y_AXIS,bPercent), aNumStr, &pTextColor);

								if( pTextColor )			// BM #60999#
								{
									pOldTextColor = (SvxColorItem*)(aYTextAttr.GetItem( EE_CHAR_COLOR ));
									aYTextAttr.Put(SvxColorItem( *pTextColor ));
								}

								// erzeuge das 3D-textobjekt
								SdrTextObj *pTextObj = CreateTextObj(CHOBJID_TEXT, Point (),
									aNumStr,
									aYTextAttr, FALSE,CHADJUST_CENTER_RIGHT);

								if( pOldTextColor )
								{
									aYTextAttr.Put( *pOldTextColor );
								}

								E3dLabelObj *pE3DLabel = new E3dLabelObj(aLine3D [1],pTextObj);
								aYDescrList.Insert(pE3DLabel,LIST_APPEND);
							}

						}

						IncValue(fAct, fValueStepMainY, bYLogarithm);
					}

				fAct = fMinValueY;
				double	fActMain = fAct;	//	This is used for calculating the positions of main grid lines.
				
				if (pXGridHelpGroup && (fMinValueY != fMaxValueY))
				{
					IncValue(fAct, fValueStepHelpY, FALSE);
					if (bSwitchColRow)
					{
						for (;;)
						{
							double fPos = pChartYAxis->CalcFact(fAct ) * aSizeVec.getX();
							if (fPos > aSizeVec.getX()) break;

							//	If there is a main grid then ommit creating help grid lines coinciding with main grid lines.
							if (pXGridMainGroup)
							{
								//	Advance main raw y position to at least the raw help position.
								while (fActMain < fAct-EPSILON)
									IncValue(fActMain, fValueStepMainY, bYLogarithm);
								//	If both positions fall together, then do not create the help line and try the next position.
								if (fabs(fAct - fActMain) < EPSILON)
								{
									IncValue(fAct, fValueStepHelpY, FALSE);
									continue;
								}
							}

							aLine3D[0].setX(fPos);
							aLine3D[1].setX(fPos);
							::basegfx::B3DPolygon aTempPoly;
							aTempPoly.append(aLine3D[0]);
							aTempPoly.append(aLine3D[1]);
							Create3DPolyObject (pXGridHelpAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_X_GRID_HELP, pXGridHelpGroup);

							//	In case of a help grid with a main grid, draw only every other line.
							IncValue(fAct, fValueStepHelpY, FALSE);
						}
					}
					else
					{
						for (;;)
						{
							double fPos = pChartYAxis->CalcFact(fAct ) * aSizeVec.getY();
							if (fPos > aSizeVec.getY()) break;

							//	If there is a main grid then ommit creating help grid lines coinciding with main grid lines.
							if (pXGridMainGroup)
							{
								//	Advance main raw y position to at least the raw help position.
								while (fActMain < fAct-EPSILON)
									IncValue(fActMain, fValueStepMainY, bYLogarithm);
								//	If both positions fall together, then do not create the help line and try the next position.
								if (fabs(fAct - fActMain) < EPSILON)
								{
									IncValue(fAct, fValueStepHelpY, FALSE);
									continue;
								}
							}

							aLine3D[0].setY(fPos);
							aLine3D[1].setY(fPos);
							::basegfx::B3DPolygon aTempPoly;
							aTempPoly.append(aLine3D[0]);
							aTempPoly.append(aLine3D[1]);
							Create3DPolyObject (pXGridHelpAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_X_GRID_HELP, pXGridHelpGroup);

							//	In case of a help grid with a main grid, draw only every other line.
							IncValue(fAct, fValueStepHelpY, FALSE);
						}
					}
				}
				break;
			}

			case 2:		// floor
			{
				// BM: create 'thick' floor
				const ::basegfx::B2DRange aPolyRect( 0.0, 0.0, aSizeVec.getX(), aSizeVec.getZ() );
				::basegfx::B2DPolyPolygon aPolyPolygon;
				aPolyPolygon.append(::basegfx::tools::createPolygonFromRect(aPolyRect));

				E3dExtrudeObj* pFloorObj = new SchE3dExtrudeObj( aDefltAttr3D, aPolyPolygon, fFloorWidth );
				::basegfx::B3DHomMatrix aMatrix;
				aMatrix.rotate( 90.0 * F_PI180, 0.0, 0.0 );
				aMatrix.translate( 0.0, 0.0, aSizeVec.getZ() );
				aMatrix.translate( aPos.getX(), aPos.getY(), aPos.getZ() );

				pFloorObj->SetModel( this );
				pFloorObj->InsertUserData( new SchObjectId( CHOBJID_DIAGRAM_FLOOR ) );
				rScene.Insert3DObj( pFloorObj );

//-/				pFloorObj->NbcSetAttributes( *pDiagramFloorAttr, FALSE );
				pFloorObj->SetMergedItemSet(*pDiagramFloorAttr);

				pFloorObj->NbcSetTransform( aMatrix * aShift );

				::basegfx::B3DPoint aEndPos( aPos );
				aEndPos.setZ(aEndPos.getZ() + aSizeVec.getZ());

				::basegfx::B3DPoint aLine3D[2] = { aPos, aEndPos };

				SfxItemSet aXTextAttr ((const SfxItemSet &) *pXAxisAttr);
				aXTextAttr.Put(XLineStyleItem (XLINE_NONE));
				aXTextAttr.Put(XLineWidthItem (0));

				//#47500#
				if (pZAxisGroup && pChartZAxis->IsVisible()) 
				{
					::basegfx::B3DPolygon aTempPoly;
					aTempPoly.append(aLine3D[0]);
					aTempPoly.append(aLine3D[1]);
					Create3DPolyObject( pZAxisAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_LINE, pZAxisGroup );
				}

				if (bSwitchColRow)
				{
					double fStepMainY  = aSizeVec.getY () / nColumnCnt;
					double fStepHelpY  = fStepMainY / 2.0;

					for (i = 0; i <= nColumnCnt; i++)
					{
						// erzeuge hilfslinie
						const double fValue(aPos.getY() + fStepMainY * i);
						aLine3D[0].setY(fValue);
						aLine3D[1].setY(fValue);

						if (pYGridMainGroup) 
						{
							::basegfx::B3DPolygon aTempPoly;
							aTempPoly.append(aLine3D[0]);
							aTempPoly.append(aLine3D[1]);
							Create3DPolyObject (pYGridMainAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Y_GRID_MAIN, pYGridMainGroup);
						}

						if ((i < nColumnCnt) && pXAxisGroup && pChartXAxis->HasDescription())
						{
							// bei tiefen diagrammtypen muessen die beschriftungen auf halbem abstand
							// untergebracht sein
							::basegfx::B3DPoint	aTextPos = aLine3D[1];

							if (bPartDescr) aTextPos.setY(aTextPos.getY() + fStepHelpY / 2.0);


							SdrTextObj *pTextObj = CreateTextObj(CHOBJID_TEXT,
								Point (),ColText(i),
								aXTextAttr, FALSE,CHADJUST_CENTER_RIGHT);
							E3dLabelObj *pE3DLabel = new E3dLabelObj
								(aTextPos,pTextObj );
							pE3DLabel->SetMarkProtect(TRUE);
							aXDescrList.Insert(pE3DLabel,LIST_APPEND);
						}

						// hilfslinien koennen mit erzeugt werden
						if (pYGridHelpGroup && (i < nColumnCnt))
						{
							aLine3D[0].setY(aLine3D[0].getY() + fStepHelpY);
							aLine3D[1].setY(aLine3D[1].getY() + fStepHelpY);
							::basegfx::B3DPolygon aTempPoly;
							aTempPoly.append(aLine3D[0]);
							aTempPoly.append(aLine3D[1]);
							Create3DPolyObject (pYGridHelpAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Y_GRID_HELP, pYGridHelpGroup);
						}
					}
				}
				else
				{
					long nStepMainX  = (long) aSizeVec.getX () / nColumnCnt;
					long nStepHelpX  = nStepMainX / 2;

					for (i = 0; i <= nColumnCnt; i++)
					{
						// erzeuge hilfslinie
						const double fValue(aPos.getX() + nStepMainX * i);
						aLine3D[0].setX(fValue);
						aLine3D[1].setX(fValue);

						if (pYGridMainGroup) 
						{
							::basegfx::B3DPolygon aTempPoly;
							aTempPoly.append(aLine3D[0]);
							aTempPoly.append(aLine3D[1]);
							Create3DPolyObject (pYGridMainAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Y_GRID_MAIN, pYGridMainGroup);
						}

						if ((i < nColumnCnt) && pXAxisGroup && pChartXAxis->HasDescription())
						{
							// bei tiefen diagrammtypen muessen die beschriftungen auf halbem abstand
							// untergebracht sein
							::basegfx::B3DPoint aTextPos = aLine3D[1];

							if (bPartDescr) aTextPos.setX(aTextPos.getX() + nStepHelpX);


							SdrTextObj *pTextObj = CreateTextObj(CHOBJID_TEXT,
								Point (),ColText(i),
								aXTextAttr, FALSE,CHADJUST_TOP_RIGHT);
							E3dLabelObj *pE3DLabel = new E3dLabelObj (aTextPos,pTextObj );
							pE3DLabel->SetMarkProtect(TRUE);
							aXDescrList.Insert(pE3DLabel,LIST_APPEND);
						}

						// hilfslinien koennen mit erzeugt werden
						if (pYGridHelpGroup && (i < nColumnCnt))
						{
							aLine3D[0].setX(aLine3D[0].getX() + nStepHelpX);
							aLine3D[1].setX(aLine3D[1].getX() + nStepHelpX);
							::basegfx::B3DPolygon aTempPoly;
							aTempPoly.append(aLine3D[0]);
							aTempPoly.append(aLine3D[1]);
							Create3DPolyObject (pYGridHelpAttr, new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Y_GRID_HELP, pYGridHelpGroup);
						}
					}
				}

				aLine3D[ 0 ] =
				aLine3D[ 1 ] = aEndPos;
				aLine3D[ 1 ].setX(aLine3D[ 1 ].getX() + aSizeVec.getX());
				
				SfxItemSet aZTextAttr ((const SfxItemSet &) *pZAxisAttr);
				aZTextAttr.Put(XLineStyleItem (XLINE_NONE));
				aZTextAttr.Put(XLineWidthItem (0));

				long nStepMainZ = (long) aSizeVec.getZ () / nRowCnt;
				long nStepHelpZ = nStepMainZ / 2;

				BOOL bCreateGridLine = ( (aLine3D[ 0 ].getX() != aLine3D[ 1 ].getX()) ||
										 (aLine3D[ 0 ].getY() != aLine3D[ 1 ].getY()) );
				// Z() values become equal in the for loop
				// => start and end points would be equal

				for (i = 0; i <= nRowCnt; i++)
				{
					// create main gridline
					const double fValue(aPos.getZ() + nStepMainZ * i);
					aLine3D[0].setZ(fValue);
					aLine3D[1].setZ(fValue);

					if( pZGridMainGroup && bCreateGridLine )
					{
						::basegfx::B3DPolygon aTempPoly;
						aTempPoly.append(aLine3D[0]);
						aTempPoly.append(aLine3D[1]);
						Create3DPolyObject( pZGridMainAttr, new SchE3dPolygonObj( aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Z_GRID_MAIN, pZGridMainGroup );
					}

					// bei tiefen diagrammtypen muessen die beschriftungen auf halbem abstand
					// untergebracht sein
					::basegfx::B3DPoint aTextPos = aLine3D[1];

					if (bPartDescr) aTextPos.setZ(aTextPos.getZ() + nStepHelpZ);

					if ((i < nRowCnt) && (nRowCnt > 1) && pZAxisGroup && pChartZAxis->HasDescription())
					{

						SdrTextObj *pTextObj=CreateTextObj(CHOBJID_TEXT, Point (),
							RowText(nRowCnt - 1 - i), aZTextAttr,
							FALSE, CHADJUST_TOP_LEFT) ;

						E3dLabelObj *pE3DLabel = new E3dLabelObj(aTextPos,pTextObj);
						pE3DLabel->SetMarkProtect(TRUE);
						aZDescrList.Insert(pE3DLabel,(ULONG)0);//ZListe umgekehrt fuellen
					}

					if (pZGridHelpGroup && (i < nRowCnt))
					{
						aLine3D[0].setZ(aLine3D[0].getZ() + nStepHelpZ);
						aLine3D[1].setZ(aLine3D[1].getZ() + nStepHelpZ);
						if( pZGridHelpGroup && bCreateGridLine )
						{
							::basegfx::B3DPolygon aTempPoly;
							aTempPoly.append(aLine3D[0]);
							aTempPoly.append(aLine3D[1]);
							Create3DPolyObject( pZGridHelpAttr, new SchE3dPolygonObj( aDefltAttr3D, ::basegfx::B3DPolyPolygon(aTempPoly), true), CHOBJID_DIAGRAM_Z_GRID_HELP, pZGridHelpGroup );
						}
					}
				}

				break;
			}
		}
	}

	pScene->InsertAllTitleText (aXDescrList, pXAxisGroup,SCH_AXIS_ID_X);
	pScene->InsertAllTitleText (aYDescrList, pYAxisGroup,SCH_AXIS_ID_Y);
	pScene->InsertAllTitleText (aZDescrList, pZAxisGroup,SCH_AXIS_ID_Z);

	if (pXGridHelpGroup) rScene.Insert3DObj(pXGridHelpGroup);
	if (pYGridHelpGroup) rScene.Insert3DObj(pYGridHelpGroup);
	if (pZGridHelpGroup) rScene.Insert3DObj(pZGridHelpGroup);
	if (pXGridMainGroup) rScene.Insert3DObj(pXGridMainGroup);
	if (pYGridMainGroup) rScene.Insert3DObj(pYGridMainGroup);
	if (pZGridMainGroup) rScene.Insert3DObj(pZGridMainGroup);
	if (pXAxisGroup) rScene.Insert3DObj(pXAxisGroup);
	if (pYAxisGroup) rScene.Insert3DObj(pYAxisGroup);
	if (pZAxisGroup) rScene.Insert3DObj(pZAxisGroup);

	//TVM: aus FitInSnapRect
	//TVM: Bound statt Logic
	Position3DAxisTitles(rScene.GetCurrentBoundRect());
}

/*************************************************************************
|*
|* tiefes 3D-Diagramme erzeugen
|*
\************************************************************************/

void impSecureIndexExistence3(::basegfx::B2DPolygon& rPoly, sal_uInt32 nIndex)
{
	if(nIndex >= rPoly.count())
	{
		rPoly.append(::basegfx::B2DPoint(), (nIndex - rPoly.count()) + 1L);
	}
}

SdrObjGroup* ChartModel::Create3DDeepChart(Rectangle &rRect)
{
	const long nGapWidth = 10;

	pScene = CreateScene (rRect, *mpLightVec, fSpotIntensity, aSpotColor,
						  fAmbientIntensity, aAmbientColor);

	BOOL        bSwitchColRow  = IsBar();
	E3dDefaultAttributes aDefltAttr3D;

	CreateAndInsert3DAxesTitles (rRect, bSwitchColRow);

	long	nX      = 0;
	long	nY      = 0;
	long	nW      = FIXED_SIZE_FOR_3D_CHART_VOLUME;
	long	nH      = FIXED_SIZE_FOR_3D_CHART_VOLUME;
	long    nZ      = (FIXED_SIZE_FOR_3D_CHART_VOLUME * 4) / 6;
	long	nColCnt = GetColCount();
	long	nRowCnt = GetRowCount();
	long nCol, nRow;

	::basegfx::B3DPolygon aFrontSide;
	aFrontSide.append(::basegfx::B3DPoint(), nColCnt + 2L);

	::basegfx::B3DPolygon aBackSide;
	aBackSide.append(::basegfx::B3DPoint(), nColCnt + 2L);

	::basegfx::B2DPolygon aFrontExtrude;
	//aFrontExtrude.append(::basegfx::B2DPoint(), 2L + nColCnt * 2L);

	long nGapX      = nW * nGapWidth / 1000;
	long nGapY      = nH * nGapWidth / 1000;
	long nGapZ      = nZ * nGapWidth / 1000;
	long nPartDepth = nZ / nRowCnt;
	long nPartWidth;
	long nBarWidthX;
	long nBarWidthZ = nPartDepth - nGapZ * 2;

	BOOL bLogarithm = ((const SfxBoolItem&) pChartYAxis->GetItemSet()->Get(SCHATTR_AXIS_LOGARITHM)).GetValue();

	BOOL bPartDescr;
	long nDepth = ((eChartStyle == CHSTYLE_3D_BAR) ||
				   (eChartStyle == CHSTYLE_3D_COLUMN) ||
				   (eChartStyle == CHSTYLE_3D_STRIPE) ||
				   (eChartStyle == CHSTYLE_3D_AREA))
					  ? -nZ
					  : nPartDepth;

	nPartWidth	= (eChartStyle == CHSTYLE_3D_BAR)
					  ? nH / nColCnt
					  : ((eChartStyle == CHSTYLE_3D_COLUMN)
							 ? nW / nColCnt
							 : ((nColCnt > 1)
									? nW / (nColCnt - 1)
									: nW / nColCnt));
	nBarWidthX	= (eChartStyle == CHSTYLE_3D_BAR)
					  ? nPartWidth - nGapY * 2
					  : nPartWidth - nGapX * 2;
	bPartDescr	= TRUE;

	Camera3D aCam(pScene->GetCamera());
	::basegfx::B3DPoint aCamPos(nX + nW/2, nH/2, nW/2);
	::basegfx::B3DPoint aLookAt(nX + nW/2, nH/2, nDepth/2);
	aCam.SetViewWindow(-nW/2, -nH/2, nW, nH);
	aCam.SetDefaults(aCamPos, aLookAt, 80, DEG2RAD(-(double)nZAngle / 10.0));
	aCam.Reset();
	aCam.SetProjection(eProjection);
	aCam.RotateAroundLookAt(DEG2RAD((double)nYAngle / 10.0), DEG2RAD((double)nXAngle / 10.0));
	aCam.SetAspectMapping(AS_HOLD_SIZE);
	pScene->SetCamera(aCam);

	nY = 0;

	::basegfx::B3DPoint a3DPos(nX, nY, nDepth);
	::basegfx::B3DPoint a3DSize(nW, nH, -nDepth);

	Create3DBackplanes(rRect, a3DPos, a3DSize, *pScene,
					   bPartDescr, FALSE, bLogarithm, FALSE, CHSTACK_NONE, FALSE, FALSE,
					   bSwitchColRow);

	nY     = (long)(pChartYAxis->CalcFactOrigin() * nH);
	a3DPos = ::basegfx::B3DPoint(0.0, nY, nDepth - nBarWidthZ + nPartDepth - nGapZ);

	DataDescription aDescription;

	for (nRow = nRowCnt - 1L; nRow >= 0L; nRow--)
	{
		E3dScene* pStripe   = NULL;

        // create row-groups. Insertion into main scene is done at the end of
        // the for loop (#109628#)
		E3dScene* pRowGroup = Create3DScene (CHOBJID_DIAGRAM_ROWGROUP);
        // #102789# some unset attributes may be propagated to the
        // top-level scene
        pRowGroup->SetMergedItemSet( pScene->GetMergedItemSet() );
		pRowGroup->InsertUserData(new SchDataRow((sal_uInt16)nRow));

		if ( eChartStyle == CHSTYLE_3D_STRIPE )
			pStripe = pRowGroup;

		a3DPos.setX(nX);
		a3DPos.setY(nY);

		switch(eChartStyle)
		{
			case  CHSTYLE_3D_COLUMN:

				a3DPos.setX(a3DPos.getX() + nGapX);
				break;

			case CHSTYLE_3D_BAR:

				// #67333# the y axis of bar charts is not interrested in negative values
				a3DPos.setY(nGapY); // += nGapY;
				break;

			default:

				aBackSide.setB3DPoint(0L, a3DPos);
				aFrontSide.setB3DPoint(0L, ::basegfx::B3DPoint(a3DPos.getX(), a3DPos.getY(), a3DPos.getZ() + nBarWidthZ));
				impSecureIndexExistence3(aFrontExtrude, 0L); 
				aFrontExtrude.setB2DPoint(0L, ::basegfx::B2DPoint(a3DPos.getX() , -a3DPos.getY()));
				break;
		}

		::basegfx::B3DPoint aLastValue;
		short     nPoints = 0;

		for (nCol = 0; nCol < nColCnt; nCol++)
		{
			double fData = GetData(nCol, nRow);

			SfxItemSet aDataPointAttr(GetFullDataPointAttr(nCol, nRow));

			SvxChartDataDescr eDescr = ((const SvxChartDataDescrItem&)aDataPointAttr.
										  Get(SCHATTR_DATADESCR_DESCR)).GetValue();

			BOOL bShowDataDescrLocal = (eDescr != CHDESCR_NONE) && bShowDataDescr && // bShowDataDescr is class member
                fData != DBL_MIN;
			if( bShowDataDescrLocal )
			{
				/**************************************************************
				* DataDescription erforderlich
				**************************************************************/
				
				aDescription.eDescr = eDescr;
				aDescription.bSymbol = ((const SfxBoolItem&)aDataPointAttr.
											  Get(SCHATTR_DATADESCR_SHOW_SYM)).GetValue();
			}

			if (eChartStyle == CHSTYLE_3D_COLUMN)
			{
				double fTop, fBottom;
				if (fData < fOriginY)
				{
					fTop	= fOriginY;
					fBottom = fData;
				}
				else
				{
					fTop	= fData;
					fBottom	= fOriginY;
				}

				long nTop =	Min((long)(pChartYAxis->CalcFact(fTop) * nH), nH);
				long nBottom = Max((long)(pChartYAxis->CalcFact(fBottom) * nH), 0L);

				if( fData != DBL_MIN )
				{
					long nBarHeight = nTop - nBottom + 1;
					a3DPos.setY(nBottom);

                    if( nTop > nBottom )
                        pRowGroup->Insert3DObj(Create3DBar(a3DPos, ::basegfx::B3DPoint(nBarWidthX, nBarHeight, nBarWidthZ), nCol, nRow,
                                                           aDataPointAttr, FALSE,0,pChartYAxis->CalcFactOrigin()*(double)nH,0));

					if( bShowDataDescrLocal )
					{
						aDescription.aTextPos3D      = a3DPos;
						aDescription.aTextPos3D += ::basegfx::B3DPoint(nBarWidthX / 2.0, (fData < 0.0)? 0.0 : nBarHeight, nBarWidthZ / 2.0);
						aDescription.fValue          = GetData(nCol, nRow,		// #67378#
							((aDescription.eDescr == CHDESCR_PERCENT) ||
							 (aDescription.eDescr == CHDESCR_TEXTANDPERCENT)) );
						aDescription.eAdjust         = CHADJUST_CENTER_CENTER;
					}
				}
				else if (bShowDataDescrLocal)
				{
					aDescription.fValue = DBL_MIN;
				}
			}
			else if (eChartStyle == CHSTYLE_3D_BAR)
			{
				double fRight, fLeft;
				if (fData < fOriginY)
				{
					fRight	= fOriginY;
					fLeft  = fData;
				}
				else
				{
					fRight = fData;
					fLeft	= fOriginY;
				}

				long nRight = Min((long)(pChartYAxis->CalcFact(fRight) * nW), nW);
				long nLeft  = Max((long)(pChartYAxis->CalcFact(fLeft) * nW), 0L);

                if( fData != DBL_MIN )
                {
					long nBarWidth = nRight - nLeft + 1;
					a3DPos.setX(nLeft);

                    if( nRight > nLeft )
                        pRowGroup->Insert3DObj(Create3DBar(a3DPos, ::basegfx::B3DPoint(nBarWidth, nBarWidthX, nBarWidthZ), nCol, nRow,
                                                           aDataPointAttr,FALSE,0,pChartYAxis->CalcFactOrigin()*(double)nW,0));

					if (bShowDataDescrLocal)
					{
						aDescription.aTextPos3D      = a3DPos;

						// #67379# corrected orientation of description
						aDescription.aTextPos3D += ::basegfx::B3DPoint((fData < 0.0) ? 0.0 : nBarWidth, nBarWidthX / 2.0, nBarWidthZ / 2.0);
						aDescription.fValue          = GetData(nCol, nRow,		// #67378#
															   ((aDescription.eDescr == CHDESCR_PERCENT) ||
																(aDescription.eDescr == CHDESCR_TEXTANDPERCENT)) );
						aDescription.eAdjust         = CHADJUST_CENTER_CENTER;
					}
				}
				else if (bShowDataDescrLocal)
				{
					aDescription.fValue = DBL_MIN;
				}
			}
			else
			{
				long nPos = (long)(pChartYAxis->CalcFact(fData ) * nH);
				if (nPos < 0L) nPos = 0L;
				else if (nPos > nH) nPos = nH;

				const SfxItemSet& rDataRowAttr = GetDataRowAttr(nRow);

				a3DPos.setY(nPos);

				switch (eChartStyle)
				{
					case CHSTYLE_3D_STRIPE:
					case CHSTYLE_3D_AREA:
						{
							long nRev       = nColCnt + 1L - nPoints;
							BOOL  bValidData = TRUE;

							if ((eChartStyle == CHSTYLE_3D_AREA) && (fData == DBL_MIN))
							{
								fData      = 0.0;
								bValidData = FALSE;
							}

							if (fData != DBL_MIN)
							{
								//Stripes:
								aFrontSide.setB3DPoint(nRev, ::basegfx::B3DPoint(a3DPos.getX(), a3DPos.getY(), a3DPos.getZ() + nBarWidthZ));
								aBackSide.setB3DPoint(nPoints, a3DPos);
								//Area:
								impSecureIndexExistence3(aFrontExtrude, nRev); 
								aFrontExtrude.setB2DPoint(nRev, ::basegfx::B2DPoint(a3DPos.getX(), -a3DPos.getY()));

								if (!nPoints)
								{
									//Stripes:
									aFrontSide.setB3DPoint(0L, ::basegfx::B3DPoint(a3DPos.getX(), 0.0, a3DPos.getZ() + nBarWidthZ));
									aBackSide.setB3DPoint(nColCnt + 1L, aFrontSide.getB3DPoint(0L));
									//Area:
									impSecureIndexExistence3(aFrontExtrude, 0L); 
									aFrontExtrude.setB2DPoint(0L, ::basegfx::B2DPoint(a3DPos.getX(), 0.0));
								}
								else if(eChartStyle == CHSTYLE_3D_STRIPE)
								{
									if (nColCnt > 1)
									{
										// deckel
										::basegfx::B3DPolygon aRect3D;
										aRect3D.append(aBackSide.getB3DPoint(nPoints - 1L));
										aRect3D.append(aFrontSide.getB3DPoint(nRev + 1L));
										aRect3D.append(aFrontSide.getB3DPoint(nRev));
										aRect3D.append(aBackSide.getB3DPoint(nPoints));
										
										E3dPolygonObj *pPolyObj=new SchE3dPolygonObj (aDefltAttr3D, ::basegfx::B3DPolyPolygon(aRect3D));
										pPolyObj->SetMergedItem(Svx3DDoubleSidedItem(TRUE)); //Neu 18.5.98
										
										Create3DPolyObject (&rDataRowAttr,pPolyObj,CHOBJID_AREA, pStripe);
									}
								}
								nPoints ++;
							}
							else bValidData = FALSE;

							if (nPoints && (eChartStyle==CHSTYLE_3D_AREA) && (nCol==nColCnt-1))
							{
								::basegfx::B3DPoint aFill = (fData == DBL_MIN) ? aLastValue : a3DPos;
								for (short nFill = nPoints;nFill <= nColCnt;nFill ++)
								{
									//aFrontSide wird noch fuer die Beschriftung gebraucht
									aFrontSide.setB3DPoint(nColCnt + 1L - nFill, ::basegfx::B3DPoint(aFill.getX(), 0.0, aFill.getZ() + nBarWidthZ));
									impSecureIndexExistence3(aFrontExtrude, nColCnt + 1L - nFill); 
									aFrontExtrude.setB2DPoint(nColCnt + 1L - nFill, ::basegfx::B2DPoint(aFill.getX(), 0.0));
								}

								if (nColCnt > 1)
								{
									aFrontExtrude.setClosed(true); //setB2DPoint(nColCnt * 2L, aFrontExtrude.getB2DPoint(0L));
									E3dExtrudeObj* pExtrudeObj= new SchE3dExtrudeObj(aDefltAttr3D, ::basegfx::B2DPolyPolygon(aFrontExtrude), nBarWidthZ);
									::basegfx::B3DHomMatrix aMatrix;
									aMatrix.translate(0.0, 0.0, a3DPos.getZ() );   //nDepth - nBarWidthZ + nPartDepth - nGapZ
									pExtrudeObj->NbcSetTransform(aMatrix);
									Create3DExtrudePolyObj(&rDataRowAttr,pExtrudeObj,CHOBJID_AREA,pRowGroup);
								}
							}    //Area

							if (fData == DBL_MIN) nPoints = 0;

							if (bShowDataDescrLocal)
							{
								aDescription.aTextPos3D = aFrontSide.getB3DPoint(nRev);

								if (bValidData)
								{
									if ((aDescription.eDescr == CHDESCR_PERCENT) ||
										(aDescription.eDescr == CHDESCR_TEXTANDPERCENT))
									{
										aDescription.fValue = GetData (nCol, nRow, TRUE, TRUE);
									}
									else
									{
										aDescription.fValue = GetData (nCol, nRow);
									}
								}
								else
								{
									aDescription.fValue = DBL_MIN;
								}

								aDescription.eAdjust = CHADJUST_CENTER_CENTER;
							}

							break;
						}

					case CHSTYLE_3D_SURFACE:
						DBG_ERROR("CHSTYLE_3D_SURFACE is not supported" );
//						if (nColCnt > 1)
//						{
//							::basegfx::B3DPolygon aTriangle3D;
//							::basegfx::B3DPolyPolygon aTempPolyPolygon;
//
//							aTriangle3D.append(a3DPos);
//							const ::basegfx::B3DPoint aPos1(a3DPos.getX(), pChartYAxis->CalcFact(GetData(nCol, nRow-1)) * nH, a3DPos.getZ() + nPartDepth);
//							aTriangle3D.append(aPos1);
//							const ::basegfx::B3DPoint aPos2(aPos1.getX() + nPartWidth, pChartYAxis->CalcFact(GetData(nCol+1, nRow-1)) * nH, aPos1.getZ());
//							aTriangle3D.append(aPos2);
//							const ::basegfx::B3DPoint aPos3(aPos2.getX(), pChartYAxis->CalcFact(GetData(nCol+1, nRow)) * nH, aPos2.getZ() - nPartDepth);
//							aTriangle3D.append(aPos3);
//
//							aTempPolyPolygon.append(aTriangle3D);
//							Create3DPolyObject (&rDataRowAttr, new SchE3dPolygonObj (aDefltAttr3D, aTempPolyPolygon, true),
//												CHOBJID_AREA, pRowGroup);
//
//							aTriangle3D.setB3DPoint(0L, aTriangle3D.getB3DPoint(1L));
//							aTriangle3D.setB3DPoint(1L, aTriangle3D.getB3DPoint(2L));
//							aTriangle3D.setB3DPoint(2L, a3DPos);
//
//							aTempPolyPolygon.clear();
//							aTempPolyPolygon.append(aTriangle3D);
//							Create3DPolyObject (&rDataRowAttr, new SchE3dPolygonObj (aDefltAttr3D, aTempPolyPolygon, true),
//												CHOBJID_AREA, pRowGroup);
//						}
						break;
				}
			}

			aLastValue = a3DPos;
			if (eChartStyle == CHSTYLE_3D_BAR) a3DPos.setY(a3DPos.getY() + nPartWidth);
			else a3DPos.setX(a3DPos.getX() + nPartWidth);

			if( pScene && bShowDataDescrLocal )
			{
				CreateDataDescr( aDescription, nCol, nRow, NULL, FALSE, TRUE );
				aDescription.pLabelObj->SetMarkProtect( TRUE );
				E3dLabelObj* pLabel = new E3dLabelObj( aDescription.aTextPos3D,
													   aDescription.pLabelObj );
				pLabel->SetMarkProtect(TRUE);
				pScene->Insert3DObj(pLabel);
			}

		}// end for nCol

        // #109628# insert row groups into main scene only if they contain data
        // points
        if( pRowGroup->GetSubList()->GetObjCount() > 0 )
        {
            pScene->Insert3DObj( pRowGroup );
        }

		/*Dirty3D (nColCnt, nRow, TRUE, pDescription, (eDataDescr != CHDESCR_NONE) && bShowDataDescr
																			 ? pScene
																			 : NULL);
		*/
		a3DPos.setZ(a3DPos.getZ() + nPartDepth);
	}//end for nRow

	return (SdrObjGroup*) pScene;
}

/*************************************************************************
|*
|* flache 3D-Diagramme erzeugen
|*
\************************************************************************/

SdrObjGroup* ChartModel::Create3DFlatChart(Rectangle &rRect)
{
	const long nGapWidth = 10;

	pScene = CreateScene (rRect, *mpLightVec, fSpotIntensity, aSpotColor,
						  fAmbientIntensity, aAmbientColor);

	BOOL       bSwitchColRow   = IsBar();
	E3dDefaultAttributes aDefltAttr3D;

	CreateAndInsert3DAxesTitles (rRect, bSwitchColRow);

	BOOL bPercent = IsPercent();


	USHORT eStackMode;
	switch (eChartStyle)
	{
		case CHSTYLE_3D_STACKEDFLATCOLUMN:
		case CHSTYLE_3D_PERCENTFLATCOLUMN:
		case CHSTYLE_3D_STACKEDFLATBAR:
		case CHSTYLE_3D_PERCENTFLATBAR:
			eStackMode = CHSTACK_MINMAX;
			break;

		case CHSTYLE_3D_STACKEDAREA:
		case CHSTYLE_3D_PERCENTAREA:
			eStackMode = CHSTACK_OVERLAP;
			break;

		default:
			eStackMode = CHSTACK_NONE;
			break;
	}

	long	nX = 0;
	long	nY = 0;
	long	nW = FIXED_SIZE_FOR_3D_CHART_VOLUME;
	long	nH = FIXED_SIZE_FOR_3D_CHART_VOLUME;
	long	nColCnt = GetColCount();
	long	nRowCnt = GetRowCount();
	long	nCol, nRow;

	::basegfx::B3DPolygon aFrontSideTwo;
	aFrontSideTwo.append(::basegfx::B3DPoint(), nColCnt * 2L + 1L);

	long nGapX      = nW * nGapWidth / 1000;
	long nGapY      = nH * nGapWidth / 1000;
	long nGapZ      = nW * nGapWidth / 1000;
	long nPartDepth = nW / nRowCnt;
	long nPartWidth;
	long nColWidthX;
	long nBarWidthX;
	long nBarWidthZ = nPartDepth - nGapZ * 2;
	long nDepth     = -nPartDepth;

	SfxItemSet* pYAxisAttr = pChartYAxis->GetItemSet();
//	BOOL bLogarithm = ((const SfxBoolItem&) pYAxisAttr->Get(SCHATTR_Y_AXIS_LOGARITHM)).GetValue();
	BOOL bLogarithm = ((const SfxBoolItem&) pYAxisAttr->Get(SCHATTR_AXIS_LOGARITHM)).GetValue();
	BOOL bPartDescr;

	::basegfx::B2DPolygon aFrontExtrude;
	//aFrontExtrude.append(::basegfx::B2DPoint(), 1L + nColCnt * 2L);

	switch (eChartStyle)
	{
		case CHSTYLE_3D_STACKEDAREA:
		case CHSTYLE_3D_PERCENTAREA:
		{
			nPartWidth	= nW / ((nColCnt > 1) ? nColCnt - 1 : nColCnt);
			nColWidthX	= 0;
			nBarWidthX	= 0;
			bPartDescr	= FALSE;
			break;
		}
		default:
		{
			nPartWidth	= bSwitchColRow
							  ? nH / nColCnt
							  : nW / nColCnt;
			nColWidthX	= bSwitchColRow
							  ? nPartWidth - nGapY * 2
							  : nPartWidth - nGapX * 2;
			nBarWidthX	= ((eChartStyle == CHSTYLE_3D_FLATCOLUMN) || (eChartStyle == CHSTYLE_3D_FLATBAR))
							  ? nColWidthX / nRowCnt
							  : 0;
			bPartDescr	= TRUE;
			break;
		}
	}

	Camera3D aCam(pScene->GetCamera());
	::basegfx::B3DPoint aCamPos(nX + nW/2, nH/2, nW/2);
	::basegfx::B3DPoint aLookAt(nX + nW/2, nH/2, nDepth/2);
	aCam.SetViewWindow(-nW/2, -nH/2, nW, nH);
	aCam.SetDefaults(aCamPos, aLookAt, 80, DEG2RAD(-(double)nZAngle / 10.0));
	aCam.Reset();
	aCam.SetProjection(eProjection);
	aCam.RotateAroundLookAt(DEG2RAD((double)nYAngle / 10.0), DEG2RAD((double)nXAngle / 10.0));
	aCam.SetAspectMapping(AS_HOLD_SIZE);
	pScene->SetCamera(aCam);

	nY = 0;
	::basegfx::B3DPoint a3DPos(nX, nY, nDepth);
	::basegfx::B3DPoint a3DSize(nW, nH, -nDepth);

	Create3DBackplanes(rRect, a3DPos, a3DSize, *pScene,
					   bPartDescr, FALSE, bLogarithm, FALSE, eStackMode, bPercent, TRUE,
					   bSwitchColRow);

	a3DPos = ::basegfx::B3DPoint(nX, nY, nDepth - nBarWidthZ + nPartDepth - nGapZ);

	switch (eChartStyle)
	{
		case CHSTYLE_3D_STACKEDAREA:
		case CHSTYLE_3D_PERCENTAREA:
		{
			DataDescription* pDescription = NULL;

			double* fOldData = new double[nColCnt];
			a3DPos.setZ(a3DPos.getZ() + nBarWidthZ);

			for (nRow = 0; nRow < nRowCnt; nRow++)
			{
				E3dScene* pRowGroup = Create3DScene (CHOBJID_DIAGRAM_ROWGROUP);
				pScene->Insert3DObj(pRowGroup);

				pRowGroup->InsertUserData(new SchDataRow((short)nRow));

				const SfxItemSet& rDataRowAttr = GetDataRowAttr(nRow);

				for (nCol = 0; nCol < nColCnt; nCol++)
				{
					double     fData          = fabs(GetData(nCol, nRow, bPercent));
					SfxItemSet aDataPointAttr (GetFullDataPointAttr(nCol, nRow));
					BOOL       bValidData;

					if (fData == DBL_MIN)
					{
						fData      = 0.0;
						bValidData = FALSE;
					}
					else bValidData = TRUE;

					SvxChartDataDescr eDescr = ((const SvxChartDataDescrItem&)aDataPointAttr.
												  Get(SCHATTR_DATADESCR_DESCR)).GetValue();

					if( (eDescr != CHDESCR_NONE) && bShowDataDescr)
					{
						/******************************************************
						* DataDescription erforderlich
						******************************************************/
						if (!pDescription)
						{
							// DataDescription noch nicht vorhanden -> erzeugen
							pDescription = new DataDescription [nColCnt];
							ClearDataDescription(pDescription,nColCnt);
						}

						pDescription [nCol].eDescr = eDescr;
						pDescription [nCol].bSymbol = ((const SfxBoolItem&)aDataPointAttr.
													   Get(SCHATTR_DATADESCR_SHOW_SYM)).GetValue();
					}

					long nRev = nColCnt * 2 - 1 - nCol;

					if (!nRow)
					{
						a3DPos.setY(Min((long)(pChartYAxis->CalcFact(fData) * nH), nH));
						aFrontSideTwo.setB3DPoint(nRev, a3DPos);
						aFrontSideTwo.setB3DPoint(nCol, ::basegfx::B3DPoint(a3DPos.getX(), 0.0, a3DPos.getZ()));
						impSecureIndexExistence3(aFrontExtrude, nRev); 
						aFrontExtrude.setB2DPoint(nRev, ::basegfx::B2DPoint(a3DPos.getX(), -a3DPos.getY()));
						impSecureIndexExistence3(aFrontExtrude, nCol); 
						aFrontExtrude.setB2DPoint(nCol, ::basegfx::B2DPoint(a3DPos.getX(), 0.0));
					}
					else
					{
						fData = fOldData[nCol] + fData;
						a3DPos.setY(Min((long)(pChartYAxis->CalcFact(fData) * nH), nH));
						aFrontSideTwo.setB3DPoint(nCol, aFrontSideTwo.getB3DPoint(nRev));
						aFrontSideTwo.setB3DPoint(nRev, a3DPos);
						impSecureIndexExistence3(aFrontExtrude, nCol); 
						aFrontExtrude.setB2DPoint(nCol, aFrontExtrude.getB2DPoint(nRev));
						impSecureIndexExistence3(aFrontExtrude, nRev); 
						aFrontExtrude.setB2DPoint(nRev, ::basegfx::B2DPoint(a3DPos.getX(), -a3DPos.getY()));
					}

					fOldData[nCol] = fData;

					if (pDescription)
					{
						const ::basegfx::B3DPoint aBasePoint(aFrontSideTwo.getB3DPoint(nRev));
						pDescription[nCol].aTextPos3D = ::basegfx::B3DPoint(
							aBasePoint.getX(), 
							aBasePoint.getY() - ((aFrontSideTwo.getB3DPoint(nRev).getY() - aFrontSideTwo.getB3DPoint(nCol).getY()) / 2.0), 
							aBasePoint.getZ());

						if (bValidData)
						{
							if ((pDescription [nCol].eDescr == CHDESCR_PERCENT) ||
								(pDescription [nCol].eDescr == CHDESCR_TEXTANDPERCENT))
							{
								pDescription [nCol].fValue = GetData (nCol, nRow, TRUE, TRUE);
							}
							else
							{
								pDescription [nCol].fValue = GetData(nCol,nRow,FALSE);//#55586# fData;
							}
						}
						else
						{
							pDescription [nCol].fValue = DBL_MIN;
						}

						pDescription [nCol].eAdjust = CHADJUST_CENTER_CENTER;

						if( eDescr != CHDESCR_NONE && bValidData )
						{
							CreateDataDescr( pDescription[ nCol ], nCol, nRow, NULL, FALSE, TRUE );
							pDescription[ nCol ].pLabelObj->SetMarkProtect( TRUE );
							E3dLabelObj* pLabel=new E3dLabelObj( pDescription[nCol].aTextPos3D,
																 pDescription[nCol].pLabelObj );
							pLabel->SetMarkProtect(TRUE);
							pScene->Insert3DObj(pLabel);
						}
					}

					a3DPos.setX(a3DPos.getX() + nPartWidth);
				} //end for nCol

				if (nColCnt > 1)
				{
					aFrontExtrude.setClosed(true); // setB2DPoint(nColCnt * 2L, aFrontExtrude.getB2DPoint(0L));[0];//.SetClosed(TRUE);
					E3dExtrudeObj* pExtrudeObj= new SchE3dExtrudeObj(aDefltAttr3D, ::basegfx::B2DPolyPolygon(aFrontExtrude), nBarWidthZ);
					::basegfx::B3DHomMatrix aMatrix;
					aMatrix.translate(0.0, 0.0 ,nDepth - nBarWidthZ + nPartDepth - nGapZ );   //-(double)nBarWidthZ
					pExtrudeObj->NbcSetTransform(aMatrix);
					//#54870# falsche ID:CHOBJID_DIAGRAM_AREA
					Create3DExtrudePolyObj(&rDataRowAttr,pExtrudeObj,CHOBJID_AREA,pRowGroup);
				}
				a3DPos.setX(nX);
			}//end for nRow

			delete[] fOldData;
			delete[] pDescription;
			break;
		}  // end case Area's

		default:
		{
			DataDescription* pDescription = NULL;

			if (bSwitchColRow) a3DPos.setY(a3DPos.getY() + nGapY);
			else a3DPos.setX(a3DPos.getX() + nGapX);

            // #100288# same structure as all other charts (no stacked and special groups)
            // create groups for all series
            E3dScene ** pDataGroup = new E3dScene * [ nRowCnt ];

            // create 3d sub-scenes for each data series. Insertion into the
            // main scene is done at the end of the for loop (#109628#)
            for( nRow = 0; nRow < nRowCnt; nRow++ )
            {
                pDataGroup[ nRow ] = Create3DScene( CHOBJID_DIAGRAM_ROWGROUP );
                pDataGroup[ nRow ]->InsertUserData( new SchDataRow( static_cast< short >( nRow ) ));
            }

            for (nCol = 0; nCol < nColCnt; nCol++)
			{
				double fDataTop		= fOriginY;
				double fDataBottom	= fOriginY;

				//Vor-Berechnung des Maximalen/Minimalen Zeichenpunktes
				double fPreBottom=fOriginY;
				double fPreTop   =fOriginY;
				double fPreTopPos,fPreBottomPos,fPreOriPos;
				double fTop,fBottom,fMin,fMax,fMin2,fMax2;
				fMin2=fMax2=fOriginY;

                for (nRow = 0; nRow < nRowCnt; nRow++)
				{
					double fData=GetData(nCol, nRow, bPercent);
					if (fData != DBL_MIN)
					{
						if (fData < fOriginY)
						{
							fTop = fPreBottom;
							if (fTop == fOriginY)
								fPreBottom = fData;
							else
								fPreBottom += fData;
							fBottom = fPreBottom;
						}
						else
						{
							fBottom = fPreTop;
							if (fBottom == fOriginY)
								fPreTop = fData;
							else
								fPreTop += fData;
							fTop = fPreTop;
						}
					}
					if(nRow)
					{
						if(fTop<fMin)
							fMin=fTop;
						if(fBottom>fMax)
							fMax=fBottom;
					}
					else
					{
						fMin=fTop;
						fMax=fBottom;
					}
					if(fData<fOriginY)
						fMin2-=fData;  //top,left
					else
						fMax2+=fData; //right,bottom
				}

				double fR = (double)( IsBar()? nW: nH );
				fPreTopPos = pChartYAxis->CalcFact( fMin2 ) * fR;
				if( fR < fPreTopPos )
					fPreTopPos = fR;
				fPreBottomPos = pChartYAxis->CalcFact( fMax2 ) * fR;
				if( fPreBottomPos < 0.0 )
					fPreBottomPos = 0.0;
				fPreOriPos = pChartYAxis->CalcFactOrigin() * fR;

				for (nRow = 0; nRow < nRowCnt; nRow++)
				{
					double     fData     = GetData(nCol, nRow, bPercent);
					SfxItemSet aDataPointAttr(GetFullDataPointAttr(nCol, nRow));

					SvxChartDataDescr eDescr = ((const SvxChartDataDescrItem&)aDataPointAttr.
												  Get(SCHATTR_DATADESCR_DESCR)).GetValue();

					if( (eDescr != CHDESCR_NONE) && bShowDataDescr)
					{
						/******************************************************
						* DataDescription erforderlich
						******************************************************/
						if (!pDescription)
						{
							// DataDescription noch nicht vorhanden -> erzeugen
							pDescription = new DataDescription [nRowCnt];
							ClearDataDescription(pDescription,nRowCnt);
						}

						pDescription [nRow].eDescr = eDescr;
						pDescription [nRow].bSymbol = ((const SfxBoolItem&)aDataPointAttr.
													   Get(SCHATTR_DATADESCR_SHOW_SYM)).GetValue();
					}

					switch (eChartStyle)
					{
						case CHSTYLE_3D_FLATCOLUMN:
							if (fData != DBL_MIN)
							{
								double fTop;
								double fBottom;

								if (fData < fOriginY)
								{
									fTop	= fOriginY;
									fBottom = fData;
								}
								else
								{
									fTop	= fData;
									fBottom	= fOriginY;
								}

								long nBottom = Max ((long) (pChartYAxis->CalcFact(fBottom) * nH),0L);
								long nTop =	Min ((long) (pChartYAxis->CalcFact(fTop) * nH),nH);

								{
									long nBarHeight = nTop - nBottom + 1;
									a3DPos.setY(nBottom);

                                    if (nTop > nBottom)
                                        pDataGroup[ nRow ]->Insert3DObj(Create3DBar(a3DPos, ::basegfx::B3DPoint(nBarWidthX, nBarHeight, nBarWidthZ), nCol, nRow,
                                                                            aDataPointAttr,TRUE,0,fPreOriPos,0));

									if (pDescription)
									{
										pDescription [nRow].aTextPos3D      = a3DPos;
										pDescription [nRow].aTextPos3D += ::basegfx::B3DPoint(nBarWidthX / 2.0, (fData < 0.0) ? 0.0 : nBarHeight, nBarWidthZ);

										if ((pDescription [nRow].eDescr == CHDESCR_PERCENT) ||
											(pDescription [nRow].eDescr == CHDESCR_TEXTANDPERCENT))
										{
											pDescription [nRow].fValue = GetData (nCol, nRow, TRUE, TRUE);
										}
										else
										{
											pDescription [nRow].fValue = GetData(nCol,nRow,FALSE);//#55586# fData;
										}

										pDescription [nRow].eAdjust = CHADJUST_CENTER_CENTER;
									}
								}
							}
							else if (pDescription)
							{
								pDescription [nRow].fValue = DBL_MIN;
							}

							a3DPos.setX(a3DPos.getX() + nBarWidthX);
							break;

						case CHSTYLE_3D_FLATBAR:
							if (fData != DBL_MIN)
							{
								double fRight;
								double fLeft;

								if (fData < fOriginY)
								{
									fRight	= fOriginY;
									fLeft   = fData;
								}
								else
								{
									fLeft	= fOriginY;
									fRight	= fData;
								}

								long nLeft = Max ((long) (pChartYAxis->CalcFact(fLeft) * nW),0L);
								long nRight = Min ((long)(pChartYAxis->CalcFact(fRight) * nW),nW);

								{
									long nBarWidth = nRight - nLeft + 1;
									a3DPos.setX(nLeft);

                                    if (nRight > nLeft)
                                        pDataGroup[ nRow ]->Insert3DObj(Create3DBar(a3DPos, ::basegfx::B3DPoint(nBarWidth, nBarWidthX, nBarWidthZ), nCol, nRow,
                                                                            aDataPointAttr,TRUE,0,fPreOriPos,0));

									if (pDescription)
									{
										pDescription [nRow].aTextPos3D      = a3DPos;
										pDescription [nRow].aTextPos3D += ::basegfx::B3DPoint((fData < 0.0) ? 0.0 : nBarWidth, nBarWidthX / 2.0, nBarWidthZ);

										if ((pDescription [nRow].eDescr == CHDESCR_PERCENT) ||
											(pDescription [nRow].eDescr == CHDESCR_TEXTANDPERCENT))
										{
											pDescription [nRow].fValue = GetData (nCol, nRow, TRUE, TRUE);
										}
										else
										{
											pDescription [nRow].fValue = GetData(nCol,nRow,FALSE);//#55586# fData;
										}

										pDescription [nRow].eAdjust = CHADJUST_CENTER_CENTER;
									}
								}
							}
							else if (pDescription)
							{
								pDescription [nRow].fValue = DBL_MIN;
							}

							a3DPos.setY(a3DPos.getY() + nBarWidthX);
							break;

						case CHSTYLE_3D_STACKEDFLATCOLUMN:
						case CHSTYLE_3D_PERCENTFLATCOLUMN:
							if (fData != DBL_MIN)
							{
								double fTop;
								double fBottom;

								if (fData < fOriginY)
								{
									fTop = fDataBottom;
									if (fTop == fOriginY) fDataBottom = fData;
									else fDataBottom += fData;
									fBottom = fDataBottom;
								}
								else
								{
									fBottom = fDataTop;
									if (fBottom == fOriginY) fDataTop = fData;
									else fDataTop += fData;
									fTop = fDataTop;
								}

								long nTop =	Min((long)(pChartYAxis->CalcFact(fTop) * nH), nH);
								long nBottom = Max((long)(pChartYAxis->CalcFact(fBottom) * nH),0L);

								{
									long nBarHeight = nTop - nBottom + 1;
									a3DPos.setY(nBottom);

                                    if (nTop > nBottom)
                                        pDataGroup[ nRow ]->Insert3DObj(Create3DBar(a3DPos, ::basegfx::B3DPoint(nColWidthX, nBarHeight, nBarWidthZ), nCol, nRow,
                                                                            aDataPointAttr,TRUE,fPreBottomPos,fPreOriPos,fPreTopPos));

									if (pDescription)
									{
										pDescription [nRow].aTextPos3D      = a3DPos;
										pDescription [nRow].aTextPos3D += ::basegfx::B3DPoint(nColWidthX / 2.0, nBarHeight / 2.0, nBarWidthZ);

										if ((pDescription [nRow].eDescr == CHDESCR_PERCENT) ||
											(pDescription [nRow].eDescr == CHDESCR_TEXTANDPERCENT))
										{
											pDescription [nRow].fValue = GetData (nCol, nRow, TRUE, TRUE);
										}
										else
										{
											pDescription [nRow].fValue = GetData(nCol,nRow,FALSE);//#55586# fData;
										}

										pDescription [nRow].eAdjust = CHADJUST_CENTER_CENTER;
									}
								}
							}
							else if (pDescription)
							{
								pDescription [nRow].fValue = DBL_MIN;
							}
							break;

						case CHSTYLE_3D_STACKEDFLATBAR:
						case CHSTYLE_3D_PERCENTFLATBAR:
							if (fData != DBL_MIN)
							{
								double fRight;
								double fLeft;

								if (fData < fOriginY)
								{
									fRight = fDataBottom;
									if (fRight == fOriginY)	fDataBottom = fData;
									else fDataBottom += fData;
									fLeft = fDataBottom;
								}
								else
								{
									fLeft = fDataTop;
									if (fLeft == fOriginY) fDataTop = fData;
									else fDataTop += fData;
									fRight = fDataTop;
								}

								long nRight = Min((long)(pChartYAxis->CalcFact(fRight) * nW), nW);
								long nLeft = Max((long) (pChartYAxis->CalcFact(fLeft ) * nW), 0L);

								{
									long nBarWidth = nRight - nLeft + 1;
									a3DPos.setX(nLeft);

                                    if (nRight > nLeft)
                                        pDataGroup[ nRow ]->Insert3DObj(Create3DBar(a3DPos, ::basegfx::B3DPoint(nBarWidth, nColWidthX, nBarWidthZ), nCol, nRow,
                                                                            aDataPointAttr,TRUE,fPreBottomPos,fPreOriPos,fPreTopPos));

									if (pDescription)
									{
										pDescription [nRow].aTextPos3D      = a3DPos;
										pDescription [nRow].aTextPos3D += ::basegfx::B3DPoint(nBarWidth / 2.0, nColWidthX  / 2.0, nBarWidthZ);

										if ((pDescription [nRow].eDescr == CHDESCR_PERCENT) ||
											(pDescription [nRow].eDescr == CHDESCR_TEXTANDPERCENT))
										{
											pDescription [nRow].fValue = GetData (nCol, nRow, TRUE, TRUE);
										}
										else
										{
											pDescription [nRow].fValue = GetData(nCol,nRow,FALSE);//#55586# fData;
										}

										pDescription [nRow].eAdjust = CHADJUST_CENTER_CENTER;
									}
								}
							}
							else if (pDescription)
							{
								pDescription [nRow].fValue = DBL_MIN;
							}
							break;
					}
				}

				//Dirty3D (nRowCnt, nCol, TRUE, pDescription, (eDataDescr != CHDESCR_NONE)&& bShowDataDescr
				//																	 ? pScene
				//																	 : NULL);

				// BM: moved here from Dirty3D.
				if( pDescription )
				{
					for (nRow = 0; nRow < nRowCnt; nRow ++)
					{
						if (pScene && pDescription[nRow].fValue != DBL_MIN)
						{
							CreateDataDescr(pDescription[nRow], nCol, nRow, NULL, TRUE, TRUE);
							pDescription[nRow].pLabelObj->SetMarkProtect(TRUE);
							E3dLabelObj* pLabel=new E3dLabelObj( pDescription[nRow].aTextPos3D,
																 pDescription[nRow].pLabelObj );
							pLabel->SetMarkProtect(TRUE);
							pScene->Insert3DObj(pLabel);
						}
					}
				}

				if (bSwitchColRow)
				{
					a3DPos.setY(a3DPos.getY() + (0L != nBarWidthX ? nGapY * 2 : nPartWidth));
					a3DPos.setX(0.0);
				}
				else
				{
					a3DPos.setX(a3DPos.getX() + (0L != nBarWidthX ? nGapX * 2 : nPartWidth));
					a3DPos.setY(0.0);
				}
			}

            // insert the data series sub-scenes if they contain any data points
            // (inserting empty scenes may corrupt the camera parameters of the
            // main scene.  See #109628#)
            for( nRow = 0; nRow < nRowCnt; nRow++ )
            {
                // sublist always exists
                if( pDataGroup[ nRow ]->GetSubList()->GetObjCount() > 0 )
                    pScene->Insert3DObj( pDataGroup[ nRow ] );
            }

			delete[] pDescription;
            delete[] pDataGroup;

			break;
		}
	}

	return (SdrObjGroup*) pScene;
}

/*************************************************************************
|*
|* 3D-Kreisdiagramm erzeugen
|*
\************************************************************************/
void ChartModel::Segment3DDescr (DataDescription &rDescr,
				   const Rectangle &rRect,
				   long            nStartAng,
				   long            nEndAng,
				   long            nHeight,
				   double          a,
				   double          b,
				   double		   fZ)
{
	long nAngleDiff;
	long nAngleHook;

	// bestimme die Winkelhalbierenden des Segments
	nAngleDiff = (nEndAng - nStartAng) / 2;
	nAngleHook = nStartAng + nAngleDiff;
	double fFullAngle = ((double)nAngleHook / 18000.0) * F_PI;

	::basegfx::B3DPoint aPnt1(cos(fFullAngle) * (a * 1.1), sin(fFullAngle) * (a * 1.1), fZ);
	::basegfx::B3DPoint aPnt2(aPnt1);
	aPnt2.setZ(-aPnt1.getZ());
	::basegfx::B3DPoint aCenter;

	B3dTransformationSet& rSet = pScene->GetCameraSet();
	const ::basegfx::B3DRange aVolume(pScene->FitInSnapRect());
	rSet.SetDeviceVolume(aVolume, FALSE);
	::basegfx::B3DHomMatrix mTransform = pScene->GetFullTransform();
	rSet.SetObjectTrans(mTransform);

	::basegfx::B3DVector a2DCenter = rSet.ObjectToViewCoor(aCenter);
	a2DCenter.setZ(0.0);
	::basegfx::B3DVector a2DPos1(rSet.ObjectToViewCoor(aPnt1));
	a2DPos1.setZ(0.0);
	::basegfx::B3DVector a2DPos2(rSet.ObjectToViewCoor(aPnt2));
	a2DPos2.setZ(0.0);
	::basegfx::B2DPoint a2DPos;

	a2DPos1 -= a2DCenter;
	a2DPos2 -= a2DCenter;

	if(a2DPos1.getLength() > a2DPos2.getLength())
	{
		rDescr.aTextPos3D = aPnt1;
		a2DPos = ::basegfx::B2DPoint(a2DPos1.getX(), a2DPos1.getY());
	}
	else
	{
		rDescr.aTextPos3D = aPnt2;
		a2DPos = ::basegfx::B2DPoint(a2DPos2.getX(), a2DPos2.getY());
	}
	long nRealHookAngle = (36000-9000+(long)RAD2CDEG(atan2(a2DPos.getX(),a2DPos.getY())))%36000;
	const long nSectorSize=(36000/8);
	long nSector=((nRealHookAngle+nSectorSize/2)/nSectorSize);///In 8 Sektoren aufteilen

	switch(nSector)
	{
		default:
		case 0:
			rDescr.eAdjust=CHADJUST_CENTER_LEFT;
			CHART_TRACE( "Segment3DDescr CHADJUST_CENTER_LEFT");
			break;
		case 1:
			rDescr.eAdjust=CHADJUST_BOTTOM_LEFT;
			CHART_TRACE( "Segment3DDescr CHADJUST_BOTTOM_LEFT");
			break;
		case 2:
			rDescr.eAdjust=CHADJUST_BOTTOM_CENTER;
			CHART_TRACE( "Segment3DDescr CHADJUST_BOTTOM_CENTER");
			break;
		case 3:
			rDescr.eAdjust=CHADJUST_BOTTOM_RIGHT;
			CHART_TRACE( "Segment3DDescr CHADJUST_BOTTOM_RIGHT");
			break;
		case 4:
			rDescr.eAdjust=CHADJUST_CENTER_RIGHT;
			CHART_TRACE( "Segment3DDescr CHADJUST_CENTER_RIGHT");
			break;
		case 5:
			rDescr.eAdjust=CHADJUST_TOP_RIGHT;
			CHART_TRACE( "Segment3DDescr CHADJUST_TOP_RIGHT");
			break;
		case 6:
			rDescr.eAdjust=CHADJUST_TOP_CENTER;
			CHART_TRACE( "Segment3DDescr CHADJUST_TOP_CENTER");
			break;
		case 7:
			rDescr.eAdjust=CHADJUST_TOP_LEFT;
			CHART_TRACE( "Segment3DDescr CHADJUST_TOP_LEFT");
			break;
	}
	a2DPos += ::basegfx::B2DPoint(a2DCenter.getX(), a2DCenter.getY());
	rDescr.aTextPos2D = a2DPos;
	Rectangle aObjRect = rDescr.pLabelObj->GetLogicRect();
	aObjRect.SetPos(Point(FRound(a2DPos.getX()), FRound(a2DPos.getY())));
	AdjustRect(aObjRect, rDescr.eAdjust);
	rDescr.pLabelObj->NbcSetAnchorPos(Point(FRound(a2DPos.getX()), FRound(a2DPos.getY())));
	rDescr.pLabelObj->NbcSetRelativePos(aObjRect.TopLeft());
	rDescr.pLabelObj->NbcSetLogicRect(aObjRect);
}

SdrObjGroup* ChartModel::Create3DNewPieChart(Rectangle &rRect)
{
	SdrObjList  *pList=NULL;

	pChartYAxis->SetPercentMode( FALSE );	// percent is calculated using the row rather than the column

	pScene = CreateScene (rRect, *mpLightVec, fSpotIntensity, aSpotColor,
						  fAmbientIntensity, aAmbientColor);
	const long nSize=FIXED_SIZE_FOR_3D_CHART_VOLUME;

	long	nW      = nSize;
	long	nH      = nSize;
	long    nZ      = nSize;//(nSize * 4) / 6;

	//erweiterung auf Donuts  emoeglichen!
	long nColCnt = GetColCount();
	const short nRowCnt=1;//	long	nRowCnt = GetRowCount();
	const short nRow=0;
	short	nCol;//, nRow;

	long nZExtrude=nZ/3;

	Camera3D aCam(pScene->GetCamera());
	const long nDepth=-nZ;

	::basegfx::B3DPoint aCamPos(0,0,nW/2);
	::basegfx::B3DPoint aLookAt(0,0,nDepth/2);
	aCam.SetViewWindow(-nW/2, -nH/2, nW, nH);
	aCam.SetDefaults(aCamPos, aLookAt, 80, DEG2RAD(-(double)nZAngle / 10.0));
	aCam.Reset();
	aCam.SetProjection(eProjection);

	aCam.SetAspectMapping(AS_HOLD_SIZE);
	pScene->SetCamera(aCam);
	pScene->SetTransform(aSceneMatrix);

	//	Pie charts may not have titles of axes.
	bShowXAxisTitle = FALSE;
	bShowYAxisTitle = FALSE;
	bShowZAxisTitle = FALSE;

	// Max. bestimmen
	double fTotal=0.0;
	long nSegments=0;
	for (nCol = 0; nCol < nColCnt; nCol++)
	{
		double fTemp = fabs(GetData(nCol,nRow));
		if (fTemp != DBL_MIN && fTemp>0)
		{
			fTotal += fTemp;
			nSegments++;
		}
	}
	if(!nSegments || fTotal == 0.0)
		return (SdrObjGroup*) pScene;

	Rectangle aPieRect;
	aPieRect=Rectangle(Point(-nW/2,-nH/2),Size(nW,nH)); // rect in which the pie is drawn

	const long nMaxAngle=36000;
	long nPos=0;
	E3dDefaultAttributes aDefltAttr3D;

	long nEndAngle=0;
	long nStartAngle=0;

	ChartDataDescription aDescr(nColCnt,nRowCnt,pList,this,bShowDataDescr);

	const ::basegfx::B2DPoint aCenter(aPieRect.Center().X(), aPieRect.Center().Y());
	Size aPieRectSize = aPieRect.GetSize();

	for (nCol = 0; nCol < nColCnt; nCol++)
	{
		double fValue = fabs(GetData( nCol,nRow));
		if(fValue!=DBL_MIN && fValue>0.0)
		{
			nPos++;
			SfxItemSet aDataPointAttr(GetFullDataPointAttr(nCol,nRow));
			nStartAngle=nEndAngle;
			nEndAngle+=(long)((fValue/fTotal)*nMaxAngle);

			// for last segment use exact value to avoid accumulated errors
			if( nPos == nSegments )
				nEndAngle = 36000;

			if(nEndAngle>nMaxAngle)
				nEndAngle-=nMaxAngle;

			::basegfx::B2DPolyPolygon aPolyPolygon;

			// if the angle of the sector is too small the conversion method produces an error
			// especially as angles are rounded to integers / 10
			if( nEndAngle - nStartAngle < 10 )	// approximate as triangle
			{
				::basegfx::B2DPolygon aPoly;
				double fAngleStart = (double)(nStartAngle) * F_PI / 18000.0,
					fAngleEnd = (double)(nEndAngle) * F_PI / 18000.0,
					fRadiusX = (double)(aPieRectSize.Width()) / 2.0,
					fRadiusY = (double)(aPieRectSize.Height()) / 2.0;

				aPoly.append(aCenter);
				aPoly.append(::basegfx::B2DPoint(aCenter.getX() + fRadiusX * cos( fAngleStart ), aCenter.getY() - fRadiusY * sin( fAngleStart )));
				aPoly.append(::basegfx::B2DPoint(aCenter.getX() + fRadiusX * cos( fAngleEnd ), aCenter.getY() - fRadiusY * sin( fAngleEnd )));
				aPoly.setClosed(true);

				aPolyPolygon.append( aPoly );
			}
			else		// create a polygon
			{
				SdrCircObj aSegment( OBJ_SECT, aPieRect, nStartAngle, nEndAngle);
				GetPage(0)->NbcInsertObject( &aSegment, 0 );
				SdrPathObj* pTmp = (SdrPathObj*)aSegment.ConvertToPolyObj( FALSE, FALSE );

				// Add two extra points near the end of the arc so that
				// the lighting of the 3d object is smoothed
				::basegfx::B2DPolygon aPoly( pTmp->GetPathPoly().getB2DPolygon(0L) );

				const sal_uInt32 nEnd(aPoly.count());

				if( nEnd > 3L )
				{
					::basegfx::B2DPoint aP2b( aPoly.getB2DPoint( nEnd-3 ) );
					::basegfx::B2DPoint aP1b( aPoly.getB2DPoint( 2 ) );
					::basegfx::B2DPoint aP2(  aPoly.getB2DPoint( nEnd-2 ) ) ;
					::basegfx::B2DPoint aP1(  aPoly.getB2DPoint( 1 ) );
					aP1 += (aP1b - aP1) / 100.0;
					aP2 += (aP2b - aP2) / 100.0;
					aPoly.insert( nEnd - 2L, aP2 );
					aPoly.insert( 2L, aP1 );
				}

				aPolyPolygon.append( aPoly );

				GetPage( 0 )->RemoveObject( 0 );

                delete pTmp;
			}

			SchE3dExtrudeObj* pObj=new SchE3dExtrudeObj(aDefltAttr3D,
				aPolyPolygon,nZExtrude);

			DBG_ASSERT( pObj, "couldn't create extrude object" );

			// default attributes reset the texture projection items so set them explicitly
			// use object specific projection in y direction
			pObj->SetMergedItem( Svx3DTextureProjectionYItem( 0 ));
			pObj->SetMergedItem( Svx3DDoubleSidedItem( TRUE ));
 
			pScene->Insert3DObj(pObj);
			pObj->InsertUserData(new SchDataPoint(nCol, nRow));
			pObj->InsertUserData (new SchObjectId (CHOBJID_DIAGRAM_DATA));
			pObj->SetMoveProtect(TRUE);
			pObj->SetResizeProtect(TRUE);
			pObj->SetModel(this);

			pObj->SetMergedItemSet(aDataPointAttr);


			::basegfx::B3DHomMatrix aMatrix;
			aMatrix.translate(0.0, 0.0, -nZExtrude/2.0);
			pObj->NbcSetTransform(aMatrix);


			if(aDescr.Enabled())
			{
				double fZPos = (double)nZExtrude / 2.0;
				DataDescription* pDescr=aDescr.Insert(nCol,nRow,aDataPointAttr,::basegfx::B2DPoint(0.0,0.0),FALSE,CHADJUST_BOTTOM_CENTER,pChartYAxis);
				if(pDescr)
					Segment3DDescr(*pDescr,aPieRect,nStartAngle,nEndAngle,0,aPieRect.GetWidth()/2,aPieRect.GetWidth()/2,fZPos);
			}
		}
	}
	aDescr.Build3D(pScene);

	return (SdrObjGroup*) pScene;
}

/*************************************************************************
|*
|* Positioniere die Achsentitel um das Rectangle der Scene
|*
\************************************************************************/

void ChartModel::Position3DAxisTitles(const Rectangle& rSceneRect)
{
	SdrPage *pPage = GetPage (0);
	Size aPageSize = pPage->GetSize();

	if (bShowXAxisTitle)
	{
		SdrObject *pXAxisTitleObj = GetObjWithId(CHOBJID_DIAGRAM_TITLE_X_AXIS, *pPage);
		if (pXAxisTitleObj != NULL)
		{
			Rectangle aXAxisOutRect = pXAxisTitleObj->GetCurrentBoundRect();

			Point aXAxesTitlePosition (rSceneRect.Left () + (int) (rSceneRect.GetWidth () / 2),
									   Min ((long) aChartRect.Bottom(),
											(long) (rSceneRect.Bottom () + aXAxisOutRect.GetHeight()) ));

			if (GetXAxisTitleHasBeenMoved() && GetUseRelativePositions() &&
				(aXAxesTitlePosition.X() > 0) && (aXAxesTitlePosition.Y() > 0))
			{
				// FG: Das ist eine Variable die in BuildChart gesetzt wird, kutz bevor
				//     das Objekt zerstoert wird.
				double fRelativeXPosition = ((double) aTitleXAxisPosition.X()) / aInitialSizefor3d.Width();
				double fRelativeYPosition = ((double) aTitleXAxisPosition.Y()) / aInitialSizefor3d.Height();
				aXAxesTitlePosition.X() = (long)(aPageSize.Width() * fRelativeXPosition);
				aXAxesTitlePosition.Y() = (long)(aPageSize.Height() * fRelativeYPosition);
			}
			else
			{
				if (bSwitch3DColRow)
				{
					aXAxesTitlePosition.X() = Max (0l, (long) (rSceneRect.Left () - 2 * aXAxisOutRect.GetWidth ()));
					aXAxesTitlePosition.Y() = Max (0l, (long) (rSceneRect.Top () + rSceneRect.GetHeight () / 2));
				}
			}
			SetTextPos((SdrTextObj &) *pXAxisTitleObj, aXAxesTitlePosition,pXAxisTitleAttr);
		}
	}

	if (bShowYAxisTitle)
	{
		SdrObject *pYAxisTitleObj = GetObjWithId(CHOBJID_DIAGRAM_TITLE_Y_AXIS, *pPage);
		if (pYAxisTitleObj != NULL)
		{
			Rectangle aYAxisOutRect = pYAxisTitleObj->GetCurrentBoundRect();

			Point aYAxesTitlePosition (Max (1l, (long) (rSceneRect.Left () - 2 * aYAxisOutRect.GetWidth ())),
									   Max (1l, (long) (rSceneRect.Top () + rSceneRect.GetHeight () / 2)));

			if (GetYAxisTitleHasBeenMoved() && GetUseRelativePositions() &&
				(aYAxesTitlePosition.X() >= 0) && (aYAxesTitlePosition.Y() > 0))
			{
				// FG: Das ist eine Variable die in BuildChart gesetzt wird, kutz bevor
				//     das Objekt zerstoert wird.
				double fRelativeXPosition = ((double) aTitleYAxisPosition.X()) / aInitialSizefor3d.Width();
				double fRelativeYPosition = ((double) aTitleYAxisPosition.Y()) / aInitialSizefor3d.Height();
				aYAxesTitlePosition.X() = (long)(aPageSize.Width() * fRelativeXPosition);
				aYAxesTitlePosition.Y() = (long)(aPageSize.Height() * fRelativeYPosition);
			}
			else
			{
				if (bSwitch3DColRow)
				{
					aYAxesTitlePosition.X() = rSceneRect.Left () + (int) (rSceneRect.GetWidth () / 2 + 0.5);
					aYAxesTitlePosition.Y() = Min ((long) aChartRect.Bottom(),
												   (long) (rSceneRect.Bottom () + aYAxisOutRect.GetHeight()) );
				}
			}
			SetTextPos((SdrTextObj &) *pYAxisTitleObj, aYAxesTitlePosition,pYAxisTitleAttr);
		}

	}

	if (bShowZAxisTitle)
	{
		SdrObject *pZAxisTitleObj = GetObjWithId(CHOBJID_DIAGRAM_TITLE_Z_AXIS, *pPage);
		if (pZAxisTitleObj != NULL)
		{
			Rectangle aZAxisOutRect = pZAxisTitleObj->GetCurrentBoundRect();


			Point aZAxesTitlePosition( (long)(0.95 * rSceneRect.Right()),
                                       (long)Min ((long) aChartRect.Bottom(),
                                                  (long) (rSceneRect.Bottom () - aZAxisOutRect.GetHeight())));


			if (GetZAxisTitleHasBeenMoved() && GetUseRelativePositions() &&
				(aZAxesTitlePosition.X() > 0) && (aZAxesTitlePosition.Y() > 0))
			{
				// FG: Das ist eine Variable die in BuildChart gesetzt wird, kutz bevor
				//     das Objekt zerstoert wird.
				double fRelativeXPosition = ((double) aTitleZAxisPosition.X()) / aInitialSizefor3d.Width();
				double fRelativeYPosition = ((double) aTitleZAxisPosition.Y()) / aInitialSizefor3d.Height();
				aZAxesTitlePosition.X() = (long)(aPageSize.Width() * fRelativeXPosition);
				aZAxesTitlePosition.Y() = (long)(aPageSize.Height() * fRelativeYPosition);
			}
			else if(aZAxesTitlePosition.Y()<aZAxisOutRect.GetHeight()/2)
			{
				aZAxesTitlePosition.Y()=aZAxisOutRect.GetHeight();
			}
			SetTextPos((SdrTextObj &) *pZAxisTitleObj, aZAxesTitlePosition,pZAxisTitleAttr);
		}

	}

	aInitialSizefor3d = aInitialSize;
}

void ChartModel::CopySpecialPointAttrToPage( long nRow )
{
	//ToDo: Kann das hier nicht mittlerweile durch CopyPointAttrToPage esetzt werden?
	SdrObjListIter aIterator(*GetPage(0), IM_DEEPWITHGROUPS);
	while (aIterator.IsMore())
	{
		SdrObject* pObj = aIterator.Next();
		SchObjectId* pObjId = GetObjectId(*pObj);

		if (pObjId && (pObjId->GetObjId() == CHOBJID_DIAGRAM_SPECIAL_GROUP))
		{
			if (nRow == GetDataRow(*pObj)->GetRow())
			{
				SchDataPoint* pDataPoint = GetDataPoint(*pObj);

				if(pDataPoint)

//-/					pObj->NbcSetAttributes(GetFullDataPointAttr(pDataPoint->GetCol(),nRow),FALSE);
					pObj->SetMergedItemSet(GetFullDataPointAttr(pDataPoint->GetCol(),nRow));

				else

//-/					pObj->NbcSetAttributes(GetDataRowAttr(nRow),FALSE);
					pObj->SetMergedItemSet(GetDataRowAttr(nRow));

			}
		}
	}
}

// eof
