/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: tpdefcol.cxx,v $
 *
 *  $Revision: 1.8 $
 *
 *  last change: $Author: obo $ $Date: 2006/09/16 20:06:32 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

// MARKER(update_precomp.py): autogen include statement, do not remove
#include "precompiled_sch.hxx"

#ifndef INCLUDED_SVTOOLS_PATHOPTIONS_HXX
#include <svtools/pathoptions.hxx>
#endif
// header for GetAppData
#ifndef _SHL_HXX
#include <tools/shl.hxx>
#endif

#include "tpappopt.hxx"
#include "tpappopt.hrc"

#include "schdll.hxx"
#include "schresid.hxx"
#include "strings.hrc"
#include "schmod.hxx"


// ====================
// class ChartColorLB
// ====================
void ChartColorLB::Fill( const SchColorTable* pTab )
{
    if( pTab == NULL )
        return;

    long nCount = pTab->size();
	SetUpdateMode( FALSE );

	for( long i = 0; i < nCount; i++ )
	{
        Append( const_cast< XColorEntry * >( & (*pTab)[ i ] ));
	}
	SetUpdateMode( TRUE );
}


// ====================
// class SchDefaultColorOptPage
// ====================
SchDefaultColorOptPage::SchDefaultColorOptPage( Window* pParent, const SfxItemSet& rInAttrs ) :
		SfxTabPage( pParent, SchResId( TP_DEF_COLOR ), rInAttrs ),
		aGbChartColors( this, SchResId( GB_CHART_COLOR_LIST ) ),
		aLbChartColors( this, SchResId( LB_CHART_COLOR_LIST ) ),
		aGbColorBox( this, SchResId( GB_COLOR_BOX ) ),
		aValSetColorBox( this, SchResId( CT_COLOR_BOX ) ),
		aPBDefault( this, SchResId( PB_RESET_TO_DEFAULT ) )
{
	FreeResource();

	aPBDefault.SetClickHdl( LINK( this, SchDefaultColorOptPage, ResetToDefaults ) );
	aLbChartColors.SetSelectHdl( LINK( this, SchDefaultColorOptPage, ListClickedHdl ) );
	aValSetColorBox.SetSelectHdl( LINK( this, SchDefaultColorOptPage, BoxClickedHdl ) );

	aValSetColorBox.SetStyle( aValSetColorBox.GetStyle()
									| WB_VSCROLL | WB_ITEMBORDER | WB_NAMEFIELD );
	aValSetColorBox.SetColCount( 8 );
	aValSetColorBox.SetLineCount( 12 );
	aValSetColorBox.SetExtraSpacing( 0 );
	aValSetColorBox.Show();

	pColorTab = new XColorTable( SvtPathOptions().GetPalettePath());

	const SfxPoolItem* pItem = NULL;
	if( rInAttrs.GetItemState( CHATTR_DEFCOLORS, FALSE, &pItem ) == SFX_ITEM_SET )
	{
		pColorConfig = SAL_STATIC_CAST( SchColorTableItem*, pItem->Clone() );
	}
	else
	{
		SchColorTable aTable;
		aTable.useDefault();
		pColorConfig = new SchColorTableItem( CHATTR_DEFCOLORS, aTable );
		pColorConfig->SetOptions( SCH_MOD1()->GetSchOptions() );
	}

	Construct();
}

SchDefaultColorOptPage::~SchDefaultColorOptPage()
{
	delete pColorConfig;
	delete pColorTab;
}

void SchDefaultColorOptPage::Construct()
{
	if( pColorConfig )
		aLbChartColors.Fill( SAL_STATIC_CAST( SchColorTable*, pColorConfig ) );

	FillColorBox();

	aLbChartColors.SelectEntryPos( 0 );
	ListClickedHdl( &aLbChartColors );
}


SfxTabPage* __EXPORT SchDefaultColorOptPage::Create( Window* pParent, const SfxItemSet& rAttrs )
{
	return new SchDefaultColorOptPage( pParent, rAttrs );
}

BOOL __EXPORT SchDefaultColorOptPage::FillItemSet( SfxItemSet& rOutAttrs )
{
	if( pColorConfig )
		rOutAttrs.Put( *SAL_STATIC_CAST( SfxPoolItem*, pColorConfig ));

	return TRUE;
}

void __EXPORT SchDefaultColorOptPage::Reset( const SfxItemSet& rInAttrs )
{
	aLbChartColors.SelectEntryPos( 0 );
	ListClickedHdl( &aLbChartColors );
}

void SchDefaultColorOptPage::FillColorBox()
{
	if( !pColorTab ) return;

	long nCount = pColorTab->Count();
	XColorEntry* pColorEntry;

	for( long i = 0; i < nCount; i++ )
	{
		pColorEntry = pColorTab->GetColor( i );
		aValSetColorBox.InsertItem( (USHORT) i + 1, pColorEntry->GetColor(), pColorEntry->GetName() );
	}
}


long SchDefaultColorOptPage::GetColorIndex( const Color& rCol )
{
	if( pColorTab )
	{
		long nCount = pColorTab->Count();
		XColorEntry* pColorEntry;

		for( long i = nCount - 1; i >= 0; i-- )			// default chart colors are at the end of the table
		{
			pColorEntry = pColorTab->GetColor( i );
			if( pColorEntry && pColorEntry->GetColor() == rCol )
				return SAL_STATIC_CAST( XPropertyTable*, pColorTab )->Get( pColorEntry->GetName() );
		}
	}
	return -1L;
}



// --------------------
// event handlers
// --------------------

// ResetToDefaults
// ---------------

IMPL_LINK( SchDefaultColorOptPage, ResetToDefaults, void *, EMPTYARG )
{
  	SAL_STATIC_CAST( SchColorTable*, pColorConfig )->useDefault();

	aLbChartColors.Clear();
	aLbChartColors.Fill( SAL_STATIC_CAST( SchColorTable*, pColorConfig ) );

	aLbChartColors.GetFocus();

	return 0L;
}

// ListClickedHdl
// --------------

IMPL_LINK( SchDefaultColorOptPage, ListClickedHdl, ChartColorLB*,  pColorList )
{
	Color aCol = pColorList->GetSelectEntryColor();

	long nIndex = GetColorIndex( aCol );

	if( nIndex == -1 )		// not found
	{
		aValSetColorBox.SetNoSelection();
	}
	else
	{
		aValSetColorBox.SelectItem( (USHORT)nIndex + 1 );		// ValueSet is 1-based
	}

	return 0L;
}

// BoxClickedHdl
// -------------

IMPL_LINK( SchDefaultColorOptPage, BoxClickedHdl, ValueSet*, pColorBox )
{
	USHORT nIdx = aLbChartColors.GetSelectEntryPos();
	if( nIdx != LISTBOX_ENTRY_NOTFOUND )
	{
		XColorEntry aEntry( aValSetColorBox.GetItemColor( aValSetColorBox.GetSelectItemId() ),
                            aLbChartColors.GetSelectEntry() );

		aLbChartColors.Modify( & aEntry, nIdx );
		pColorConfig->replace( nIdx, aEntry );

        aLbChartColors.SelectEntryPos( nIdx );	// reselect entry
	}

    return 0L;
}

