/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: undatpnt.cxx,v $
 *
 *  $Revision: 1.5 $
 *
 *  last change: $Author: obo $ $Date: 2006/09/16 20:15:46 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

// MARKER(update_precomp.py): autogen include statement, do not remove
#include "precompiled_sch.hxx"

//svdraw.hxx
//#define _SVDRAW_HXX
#define _SDR_NOITEMS
#define _SDR_NOTOUCH
#define _SDR_NOTRANSFORM
//#define _SDR_NOOBJECTS
//#define _SDR_NOVIEWS
#define _SDR_NOVIEWMARKER
#define _SDR_NODRAGMETHODS
#define _SDR_NOUNDO
#define _SDR_NOXOUTDEV

#ifndef _SFX_OBJSH_HXX //autogen
#include <sfx2/objsh.hxx>
#endif








#ifndef _CHTMODEL_HXX
#include "chtmodel.hxx"
#endif

#include "schdll.hxx"
#include "schresid.hxx"
#include "strings.hrc"
#include "schview.hxx"
#include "datapoin.hxx"
#include "undatpnt.hxx"

TYPEINIT1(SchUndoDataPoint, SchUndoAttr);


/*************************************************************************
|*
|*
|*
\************************************************************************/

void SchUndoDataPoint::Undo()
{
	SdrObject *pObj=rDoc.GetDataPointObj(nCol, nRow);
	if(pObj)
	{
		rDoc.ChangeDataPointAttr(*pOldAttr,*pObj, FALSE);
	}
	else
	{
		DBG_ERROR("SchUndoDataPoint::Undo : Datapointobject not found");
	}
}

/*************************************************************************
|*
|*
|*
\************************************************************************/

void SchUndoDataPoint::Redo()
{
	rDoc.ChangeDataPointAttr(*pNewAttr, *rDoc.GetDataPointObj(nCol, nRow));
}

/*************************************************************************
|*
|*
|*
\************************************************************************/

void SchUndoDataPoint::Repeat(SfxRepeatTarget& rView)
{
	if (((SchView&)rView).AreObjectsMarked())
	{
		const SdrMarkList& rMarkList = ((SchView&)rView).GetMarkedObjectList();

		if (rMarkList.GetMarkCount() == 1)
		{
			SdrObject* pObj = rMarkList.GetMark(0)->GetMarkedSdrObj();
			SchDataPoint* pDataPoint = GetDataPoint(*pObj);

			if (pDataPoint)
			{
				short nCol = pDataPoint->GetCol();
				short nRow = pDataPoint->GetRow();

				SfxItemSet aOldAttr(rDoc.GetDataPointAttr(nCol, nRow));

				if (rDoc.ChangeDataPointAttr(*pNewAttr, *pObj))
				{
					SchUndoDataPoint* pUndo =
						new SchUndoDataPoint(rDoc, nCol, nRow,
											 aOldAttr, *pNewAttr);
					pUndo->SetComment(String(SchResId(STR_UNDO_DATA_POINT)));

					rDoc.GetObjectShell()->
						GetUndoManager()->AddUndoAction(pUndo);
				}
			}
		}
	}
}

/*************************************************************************
|*
|*
|*
\************************************************************************/

BOOL SchUndoDataPoint::CanRepeat(SfxRepeatTarget& rView) const
{
	if (((SchView&)rView).AreObjectsMarked())
	{
		const SdrMarkList& rMarkList = ((SchView&)rView).GetMarkedObjectList();

		if (rMarkList.GetMarkCount() == 1 &&
			GetDataPoint(*rMarkList.GetMark(0)->GetMarkedSdrObj()))
			return TRUE;
	}

	return FALSE;
}



