/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: salrectangleutils.cxx,v $
 *
 *  $Revision: 1.5 $
 *
 *  last change: $Author: obo $ $Date: 2006/09/17 11:45:50 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

// MARKER(update_precomp.py): autogen include statement, do not remove
#include "precompiled_vcl.hxx"

#ifndef _SV_SALRECTANGLEUTILS_HXX
	#include <salrectangleutils.hxx>
#endif

// =======================================================================

// =======================================================================

void CheckRectBounds ( const Rect  *rPortBoundsRect,
                       Rect        *rSrcRect, 
                       Rect        *rDstRect
                     )
{
	if ( rSrcRect->top < rPortBoundsRect->top )
	{
		rDstRect->top += (rPortBoundsRect->top - rSrcRect->top);
		rSrcRect->top  = rPortBoundsRect->top;
	} // if

	if ( rSrcRect->left < rPortBoundsRect->left )
	{
		rDstRect->left += (rPortBoundsRect->left - rSrcRect->left);
		rSrcRect->left  = rPortBoundsRect->left;
	} // if

	if ( rSrcRect->bottom > rPortBoundsRect->bottom )
	{

		rDstRect->bottom += (rPortBoundsRect->bottom - rSrcRect->bottom);
		rSrcRect->bottom  = rPortBoundsRect->bottom;
	} // if

	if ( rSrcRect->right > rPortBoundsRect->right )
	{
		rDstRect->right += (rPortBoundsRect->right - rSrcRect->right);
		rSrcRect->right  = rPortBoundsRect->right;
	} // if
} // CheckRectBounds

// =======================================================================

// =======================================================================

void SalTwoRect2QDDstRect ( const SalTwoRect  *pPosAry,
                            Rect              *rDstRect
                          )
{
	MacSetRect( rDstRect,
	            pPosAry->mnDestX,
	            pPosAry->mnDestY,
	            pPosAry->mnDestX + pPosAry->mnDestWidth,
	            pPosAry->mnDestY + pPosAry->mnDestHeight
	          );
} // GetSalSourceRect

// -----------------------------------------------------------------------

void SalTwoRect2QDSrcRect ( const SalTwoRect  *pPosAry,
                            Rect              *rSrcRect
                          )
{
	MacSetRect( rSrcRect, 
	            pPosAry->mnSrcX,
	            pPosAry->mnSrcY,
	            pPosAry->mnSrcX + pPosAry->mnSrcWidth,
	            pPosAry->mnSrcY + pPosAry->mnSrcHeight
	          );
} // SalTwoRect2QDSrcRect
				
// =======================================================================

// =======================================================================

void GetQDRect ( const long   nX, 
                 const long   nY,
                 const long   nWidth, 
                 const long   nHeight,
                 Rect        *rRect
               )
{
	MacSetRect( rRect,  nX, nY, nX + nWidth, nY + nHeight );
} // GetQDRect

// -----------------------------------------------------------------------

void GetTwoRectFromSalBmp( const SalBitmap  &rSalBmp,
                           SalTwoRect       *rTwoRect
                         )
{
	const long  nWidth  = rSalBmp.GetSize().Width();
	const long  nHeight = rSalBmp.GetSize().Height();

	// Get size of graphics to copy from

	rTwoRect->mnSrcX  = 0;
	rTwoRect->mnDestX = 0;
	rTwoRect->mnSrcY  = 0;
	rTwoRect->mnDestY = 0;

	rTwoRect->mnSrcWidth   = nWidth;
	rTwoRect->mnDestWidth  = nWidth;
	rTwoRect->mnSrcHeight  = nHeight;
	rTwoRect->mnDestHeight = nHeight;
} // GetTwoRectFromSalBmp
				
// =======================================================================

// =======================================================================

